Package: DLFM
Type: Package
Version: 0.2.4
Title: Distributed Laplace Factor Model
Authors@R: 
    c(person(given = "Guangbao",
             family = "Guo",
             role = c("aut", "cre"),
             email = "ggb11111111@163.com"),
      person(given = "Siqi",
             family = "Liu",
             role = "aut"))
Description: Distributed estimation method is based on a Laplace factor model
    to solve the estimates of load and specific variance. The philosophy of
    the package is described in Guangbao Guo. (2022).
    <doi:10.1007/s00180-022-01270-z>.
License: MIT + file LICENSE
Encoding: UTF-8
RoxygenNote: 7.3.2
Imports: MASS, LaplacesDemon, matrixcalc, stats
Depends: R (>= 3.5.0)
Suggests: testthat (>= 3.0.0)
Config/testthat/edition: 3
LazyData: true
BuildManual: yes
NeedsCompilation: no
Language: en-US
Repository: CRAN
Packaged: 2026-05-13 13:47:08 UTC; R7000
Author: Guangbao Guo [aut, cre],
  Siqi Liu [aut]
Maintainer: Guangbao Guo <ggb11111111@163.com>
Date/Publication: 2026-05-18 02:30:02 UTC
Built: R 4.6.0; ; 2026-05-18 05:29:46 UTC; unix

# Generated by roxygen2: do not edit by hand

export(DGulPC)
export(DIPC)
export(DPC)
export(DPPC)
export(DSAPC)
export(Dfactor.tests)
export(FanPC)
export(Ftest)
export(GulPC)
export(IPC)
export(LFM)
export(PC)
export(PPC)
export(SAPC)
export(calculate_DAA)
export(estimate_AD_from_OSDR)
export(factor.tests)
export(frobenius.norm)
export(generate_asymmetric_laplace)
export(generate_multivariate_laplace)
export(generate_skew_laplace)
export(online_sir_lfm)
export(osdr_lfm)
importFrom(LaplacesDemon,is.positive.definite)
importFrom(LaplacesDemon,is.square.matrix)
importFrom(LaplacesDemon,is.symmetric.matrix)
importFrom(LaplacesDemon,lower.triangle)
importFrom(LaplacesDemon,rallaplace)
importFrom(LaplacesDemon,rlaplace)
importFrom(LaplacesDemon,rllaplace)
importFrom(LaplacesDemon,rslaplace)
importFrom(LaplacesDemon,upper.triangle)
importFrom(MASS,ginv)
importFrom(MASS,mvrnorm)
importFrom(matrixcalc,frobenius.norm)
importFrom(stats,coef)
importFrom(stats,cor)
importFrom(stats,cov)
importFrom(stats,lm)
importFrom(stats,p.adjust)
importFrom(stats,pcauchy)
importFrom(stats,pchisq)
importFrom(stats,pf)
importFrom(stats,pnorm)
importFrom(stats,qcauchy)
importFrom(stats,qnorm)
importFrom(stats,quantile)
importFrom(stats,residuals)
importFrom(stats,rexp)
importFrom(stats,rgamma)
importFrom(stats,rnorm)
importFrom(stats,runif)

#' @name Ftest
#' @title Apply the Farmtest method to the Laplace factor model
#' @description This function simulates data from a Lapalce factor model and applies the FarmTest
#' for multiple hypothesis testing. It calculates the false discovery rate (FDR)
#' and power of the test.
#' @param data A matrix or data frame of simulated or observed data from a Laplace factor model.
#' @param p1 The number or proportion of non-zero hypotheses.
#' @param alpha The significance level for controlling the false discovery rate (default: 0.05).
#' @param K The number of factors to estimate (default: -1, meaning auto-detect).
#' @param alternative The alternative hypothesis: "two.sided", "less", or "greater" (default: "two.sided").
#' @return A list containing the following elements:
#' \item{FDR}{The false discovery rate, which is the proportion of false positives among all discoveries (rejected hypotheses).}
#' \item{Power}{The statistical power of the test, which is the probability of correctly rejecting a false null hypothesis.}
#' \item{PValues}{A vector of p-values associated with each hypothesis test.}
#' \item{RejectedHypotheses}{The total number of hypotheses that were rejected by the FarmTest.}
#' \item{reject}{Indices of rejected hypotheses.}
#' \item{means}{Estimated means.}
#' @examples
#' library(LaplacesDemon)
#' library(MASS)
#' n=1000
#' p=10
#' m=5
#' mu=t(matrix(rep(runif(p,0,1000),n),p,n))
#' mu0=as.matrix(runif(m,0))
#' sigma0=diag(runif(m,1))
#' F=matrix(mvrnorm(n,mu0,sigma0),nrow=n)
#' A=matrix(runif(p*m,-1,1),nrow=p)
#' lanor <- rlaplace(n*p,0,1)
#' epsilon=matrix(lanor,nrow=n)
#' D=diag(t(epsilon)%*%epsilon)
#' data=mu+F%*%t(A)+epsilon
#' p1=40
#' results <- Ftest(data, p1)
#' print(results$FDR)
#' print(results$Power)
#' @export
Ftest <- function(data, p1, alpha = 0.05, K = -1, alternative = c("two.sided", "less", "greater")) {
  
  alternative <- match.arg(alternative)
  
  # Check if p1 is proportion or count
  if (p1 < 1 && p1 > 0) {
    # p1 is a proportion, convert to count
    p1_count <- round(p1 * ncol(data))
  } else {
    # p1 is a count
    p1_count <- p1
  }
  
  # Apply FarmTest (simplified version)
  output <- simplified_farm_test(data, alpha, K, alternative)
  
  # Calculate FDR and power based on p1_count non-zero hypotheses
  # Assuming first p1_count hypotheses are non-zero (for simulation purposes)
  n_hypotheses <- ncol(data)
  
  if (p1_count > n_hypotheses) {
    p1_count <- n_hypotheses
  }
  
  # Get rejected hypotheses indices
  rejected_indices <- output$reject
  n_rejected <- length(rejected_indices)
  
  # Calculate FDR and Power
  if (n_rejected > 0) {
    # False discoveries: rejected among the null hypotheses (indices > p1_count)
    false_discoveries <- sum(rejected_indices > p1_count)
    fdr <- false_discoveries / n_rejected
    
    # True discoveries: rejected among the alternative hypotheses (indices <= p1_count)
    true_discoveries <- sum(rejected_indices <= p1_count)
    power <- true_discoveries / p1_count
  } else {
    fdr <- 0
    power <- 0
  }
  
  # Return the results as a list
  return(list(
    FDR = fdr,
    Power = power,
    PValues = output$pValues,
    RejectedHypotheses = n_rejected,
    reject = rejected_indices,
    means = output$means
  ))
}

# Simplified implementation of farm.test
simplified_farm_test <- function(X, alpha = 0.05, K = -1, 
                                 alternative = c("two.sided", "less", "greater")) {
  
  alternative <- match.arg(alternative)
  n <- nrow(X)
  p <- ncol(X)
  
  # Estimate means using robust Huber-like approach
  means <- apply(X, 2, function(x) {
    # Simple robust mean estimation (trimmed mean)
    stats::quantile(x, 0.5)  # Using median as robust estimator
  })
  
  # Estimate factor model if K != 0
  if (K != 0) {
    # Simple factor estimation using PCA
    if (K < 0) {
      # Auto-detect number of factors
      K <- estimate_number_of_factors(X)
    }
    
    if (K > 0 && K < min(n, p)) {
      # Perform PCA
      pca_result <- stats::prcomp(X, center = TRUE, scale. = FALSE)
      
      # Get factor loadings and scores
      loadings <- pca_result$rotation[, 1:K, drop = FALSE]
      scores <- pca_result$x[, 1:K, drop = FALSE]
      
      # Remove factor effects
      X_resid <- X - scores %*% t(loadings)
    } else {
      X_resid <- X
      loadings <- matrix(0, p, 0)
      K <- 0
    }
  } else {
    X_resid <- X
    loadings <- matrix(0, p, 0)
  }
  
  # Calculate test statistics
  if (K == 0) {
    # Without factor adjustment
    resid_means <- means
    resid_sd <- apply(X_resid, 2, function(x) {
      # Robust scale estimation using MAD
      stats::mad(x, constant = 1)
    })
  } else {
    # With factor adjustment
    resid_means <- apply(X_resid, 2, stats::median)
    resid_sd <- apply(X_resid, 2, function(x) stats::mad(x, constant = 1))
  }
  
  # Calculate t-statistics
  t_stat <- resid_means / (resid_sd / sqrt(n))
  
  # Calculate p-values based on alternative hypothesis
  if (alternative == "two.sided") {
    p_values <- 2 * (1 - stats::pt(abs(t_stat), df = n - 1))
  } else if (alternative == "less") {
    p_values <- stats::pt(t_stat, df = n - 1)
  } else { # "greater"
    p_values <- 1 - stats::pt(t_stat, df = n - 1)
  }
  
  # Adjust p-values using Benjamini-Hochberg method
  p_adjust <- stats::p.adjust(p_values, method = "BH")
  
  # Determine significant hypotheses
  significant <- as.integer(p_adjust <= alpha)
  
  # Get indices of rejected hypotheses
  reject <- which(significant == 1)
  if (length(reject) == 0) {
    reject <- "no hypotheses rejected"
  }
  
  # Return results
  return(list(
    means = means,
    stdDev = resid_sd,
    loadings = loadings,
    nFactors = K,
    tStat = t_stat,
    pValues = p_values,
    pAdjust = p_adjust,
    significant = significant,
    reject = reject,
    type = ifelse(K == 0, "no factor", "unknown"),
    n = n,
    p = p,
    h0 = rep(0, p),
    alpha = alpha,
    alternative = alternative
  ))
}

# Helper function to estimate number of factors
estimate_number_of_factors <- function(X) {
  n <- nrow(X)
  p <- ncol(X)
  
  # Simple eigenvalue ratio method
  if (n > 1 && p > 1) {
    # Center the data
    X_centered <- scale(X, scale = FALSE)
    
    # Calculate covariance matrix
    Sigma <- stats::cov(X_centered)
    
    # Eigen decomposition
    eigen_vals <- eigen(Sigma, symmetric = TRUE, only.values = TRUE)$values
    
    # Calculate ratios of consecutive eigenvalues
    ratios <- eigen_vals[-length(eigen_vals)] / eigen_vals[-1]
    
    # Estimate number of factors (simple heuristic)
    # Use eigenvalues greater than average
    avg_eigen <- mean(eigen_vals)
    K <- sum(eigen_vals > avg_eigen)
    
    # Ensure K is reasonable
    K <- min(K, floor(min(n, p) / 2))
  } else {
    K <- 0
  }
  
  return(max(0, K))
}

#' @name DPC
#' @title Distributed principal component
#' @param data is a total data set
#' @param m is the number of principal component
#' @param n1 is  the length of each data subset
#' @param K is the number of nodes
#' @return Ahat,Dhat,Sigmahathat
#' @export
#' @examples
#' library(LaplacesDemon)
#' library(MASS)
#' n=1000
#' p=10
#' m=5
#' mu=t(matrix(rep(runif(p,0,1000),n),p,n))
#' mu0=as.matrix(runif(m,0))
#' sigma0=diag(runif(m,1))
#' F=matrix(mvrnorm(n,mu0,sigma0),nrow=n)
#' A=matrix(runif(p*m,-1,1),nrow=p)
#' lanor <- rlaplace(n*p,0,1)
#' epsilon=matrix(lanor,nrow=n)
#' D=diag(t(epsilon)%*%epsilon)
#' data=mu+F%*%t(A)+epsilon
#' DPC(data,m=3,n1=128,K=2)
#' @importFrom stats cor cov
DPC=function(data,m,n1,K){
  n<-nrow(data)
  p=ncol(data)
  X1=matrix(rep(0,n1*p),ncol=p)
  Sigmahat=list()
  Ahat=list()
  Dhat=list()
  for (i in 1:K) {
    L=matrix(rep(0,K*n1),ncol=n1)
    R=matrix(0,n1,n)
    L[i,]=sample(1:n,n1,replace=FALSE)
    r=matrix(c(1:n1,L[i,]),ncol=n1,byrow=T)
    R[t(r)]=1
    X1=R%*%as.matrix(data)
    X=scale(X1)
    Sigmahat[[i]]<-cor(X)
    eig<-eigen(Sigmahat[[i]])
    lambdahat= eig$values[1:m]
    ind<-order(lambdahat,decreasing=T)
    lambdahat<-lambdahat[ind]
    Q<- eig$vectors
    Q<-Q[,ind]
    Qhat<-Q[,1:m]
    Ahat1 <- matrix(0, nrow = p, ncol = m)
    for (j in 1:m) {Ahat1[, j] <- sqrt(lambdahat[j]) * Qhat[, j]};
    Ahat[[i]] =Ahat1
    h0 <- diag(Ahat[[i]] %*% t(Ahat[[i]]))
    Dhat[[i]]<- diag(Sigmahat[[i]] - h0)
    S2=Ahat[[i]] %*% t(Ahat[[i]])+Dhat[[i]]}
  return(
    list(Ahat=Ahat,Dhat=Dhat,Sigmahat=Sigmahat))}

#' @name PC
#' @title Principal component
#' @param data is a total data set
#' @param m is the number of principal component
#' @return Ahat, Dhat, Sigmahat
#' @export
#' @examples
#' library(LaplacesDemon)
#' library(MASS)
#' n=1000
#' p=10
#' m=5
#' mu=t(matrix(rep(runif(p,0,1000),n),p,n))
#' mu0=as.matrix(runif(m,0))
#' sigma0=diag(runif(m,1))
#' F=matrix(mvrnorm(n,mu0,sigma0),nrow=n)
#' A=matrix(runif(p*m,-1,1),nrow=p)
#' lanor <- rlaplace(n*p,0,1)
#' epsilon=matrix(lanor,nrow=n)
#' D=diag(t(epsilon)%*%epsilon)
#' data=mu+F%*%t(A)+epsilon
#' PC(data,m=5)
PC=function(data,m){
  X<-scale(data)
  p<-ncol(X)
  n<-nrow(X)
  Sigmahat<-cov(X)
  eig<-eigen(Sigmahat)
  lambdahat= eig$values[1:m]
  ind<-order(lambdahat,decreasing=T)
  lambdahat<-lambdahat[ind]
  Q<- eig$vectors
  Q<-Q[,ind]
  Qhat<-Q[,1:m]
  Ahat <- matrix(0, nrow = p, ncol = m)
  for (j in 1:m) {Ahat[, j] <- sqrt(lambdahat[j]) * Qhat[, j]}; Ahat
  h0 <- diag(Ahat %*% t(Ahat))
  Dhat<- diag(Sigmahat - h0)
  S2=Ahat %*% t(Ahat)+Dhat
  return(
    list(Ahat=Ahat,Dhat=Dhat,Sigmahat=Sigmahat))}

#' @name online_sir_lfm
#' @title Online Sufficient Dimension Reduction for Laplace Factor Model (LFM)
#'
#' @description Implements an online SIR algorithm tailored for LFM data, using a proxy response constructed from the current subspace estimate and robust updates to handle heavy-tailed noise. The algorithm supports two optimization methods: gradient-based updates and perturbation-based updates.
#'
#' @param X A matrix or data stream of size n x p (rows = observations, cols = features). Can be processed row-by-row in streaming setting.
#' @param K_true Optional true dimension (for monitoring). If NULL, will estimate online via BIC-like criterion.
#' @param K_max Maximum candidate dimension for online selection (default = min(10, ncol(X))).
#' @param c_robust Robustness scale for tanh transformation (default = 1.345, approx. 0.95 efficiency for Gaussian).
#' @param eta Learning rate schedule: either a function of t, or "auto" for 1/t.
#' @param method Optimization method: "gradient" for gradient-based updates with learning rate, or "perturbation" for direct eigenvector computation of the moment matrix (default = "gradient").
#' @param verbose Logical; if TRUE, prints progress and estimated K at each step.
#'
#' @return A list with:
#' \item{B_hat}{Final estimated basis matrix (p x K_est)}
#' \item{K_est}{Estimated structural dimension}
#' \item{B_path}{List of B estimates over time (optional, for debugging)}
#' \item{loss}{Reconstruction loss trace (optional)}
#' \item{method_used}{The optimization method actually used}
#' @export
#' @examples
#' set.seed(123)
#' n <- 500; p <- 20; m <- 3
#' B_true <- qr.Q(qr(matrix(rnorm(p * m), p, m)))
#' f <- matrix(rnorm(n * m), n, m)
#' eps <- matrix(rexp(n * p, rate = 1) - 1, n, p) # Asymmetric Laplace-like noise
#' X <- f %*% t(B_true) + eps
#'
#' # Using gradient method (default)
#' out_grad <- online_sir_lfm(X, K_true = m, verbose = TRUE)
#'
#' # Using perturbation method
#' out_pert <- online_sir_lfm(X, K_true = m, method = "perturbation", verbose = TRUE)
#'
#' @importFrom stats rnorm
#'
online_sir_lfm <- function(X, K_true = NULL, K_max = NULL, c_robust = 1.345,
                           eta = "auto", method = "gradient",  
                           verbose = FALSE) {
  
  if (!is.matrix(X)) X <- as.matrix(X)
  n <- nrow(X); p <- ncol(X)
  
  if (is.null(K_max)) K_max <- min(10, p)
  if (K_max > p) K_max <- p
  
  B_hat <- qr.Q(qr(matrix(rnorm(p * K_max), p, K_max)))
  x_bar <- rep(0, p)
  M_hat <- matrix(0, p, p)
  lambda_sq_sum <- rep(0, K_max)
  
  B_path <- list()
  loss_trace <- numeric(n)
  
  if (is.character(eta) && eta == "auto") {
    get_eta <- function(t) 1 / t
  } else if (is.function(eta)) {
    get_eta <- eta
  } else {
    stop("eta must be 'auto' or a function of iteration index t")
  }
  
  valid_methods <- c("gradient", "perturbation")
  if (!method %in% valid_methods) {
    warning(paste("method", method, "not recognized. Using 'gradient'"))
    method <- "gradient"
  }
  
  for (t in 1:n) {
    x_new <- X[t, , drop = TRUE]
    
    x_bar <- ((t - 1) / t) * x_bar + (1 / t) * x_new
    
    proj <- B_hat %*% t(B_hat) %*% x_new
    y_t <- sqrt(sum(proj^2))
    
    resid <- x_new - x_bar
    r_robust <- tanh(resid / c_robust)
    
    outer_rr <- tcrossprod(r_robust)
    M_hat <- ((t - 1) / t) * M_hat + (1 / t) * outer_rr * y_t
    
    if (method == "gradient") {
      eta_t <- get_eta(t)
      grad <- M_hat %*% B_hat
      B_temp <- B_hat + eta_t * grad
      qr_temp <- qr(B_temp)
      B_hat <- qr.Q(qr_temp)
      
    } else if (method == "perturbation") {
      B_temp <- M_hat %*% B_hat
      qr_temp <- qr(B_temp)
      B_hat <- qr.Q(qr_temp)
    }
    
    Mt_proj <- t(B_hat) %*% M_hat %*% B_hat
    eig_Mt <- eigen(Mt_proj, symmetric = TRUE, only.values = TRUE)$values
    eig_Mt <- pmax(eig_Mt, 0)
    total_var <- sum(eig_Mt)
    
    if (total_var > 1e-12) {
      D_vals <- numeric(K_max)
      for (k in 1:K_max) {
        ratio <- sum(eig_Mt[1:k]^2) / total_var
        penalty <- (t^0.5) * k * (k + 1) / (2 * t)
        D_vals[k] <- ratio - penalty
      }
      K_est_t <- which.max(D_vals)
    } else {
      K_est_t <- 1
    }
    
    if (!is.null(K_true)) {
      B_trunc <- B_hat[, 1:K_est_t, drop = FALSE]
      recon <- B_trunc %*% (t(B_trunc) %*% x_new)
      loss_trace[t] <- sum((x_new - recon)^2)
    }
    
    if (verbose && (t %% 50 == 0 || t == n)) {
      cat(sprintf("Step %d: Method=%s, Est K=%d\n", t, method, K_est_t))
    }
  }
  
  final_K <- K_est_t
  B_final <- B_hat[, 1:final_K, drop = FALSE]
  
  return(list(
    B_hat = B_final,
    K_est = final_K,
    B_path = B_path,
    loss = if (!is.null(K_true)) loss_trace else NULL,
    method_used = method
  ))
}

#' @name calculate_DAA
#' @title Distance Between Column Spaces of Two Loading Matrices
#'
#' @description Computes a normalized distance between the subspaces spanned by the columns of two factor loading matrices \code{Ahat} (estimated) and \code{A1} (true/reference). The metric is based on the projection matrix of \code{A1}, regularized to avoid singularity, and is scaled by the number of factors \code{m} and squared number of variables \code{p^2} to lie in \eqn{[0,1]}.
#'
#' @param Ahat Estimated factor loading matrix of dimension \code{p x m}, or at least having \code{m} columns.
#' @param A1 True or reference factor loading matrix of dimension \code{p x m} (or \code{p x m1}, but typically \code{m} columns).
#' @param m Number of factors (structural dimension) used in the scaling factor.
#' @param p Number of variables (observed dimensions).
#'
#' @return A scalar in \eqn{[0,1]} representing the discrepancy between the column spaces of \code{Ahat} and \code{A1}. A value of 0 indicates identical subspaces; values closer to 1 indicate greater discrepancy.
#'
#' @details The distance is defined as
#' \deqn{D(A_{\mathrm{est}}, A_{\mathrm{true}}) = \sqrt{1 - \frac{1}{m p^2} \, \mathrm{Re}\left( \mathrm{tr}\left( A_{\mathrm{est}}^\top (A_{\mathrm{true}} A_{\mathrm{true}}^\top + \epsilon I)^{-1} A_{\mathrm{true}} A_{\mathrm{true}}^\top A_{\mathrm{est}} \right) \right)},}
#' where \eqn{\epsilon = 10^{-6}} is a small ridge parameter to ensure invertibility of \eqn{A_{\mathrm{true}} A_{\mathrm{true}}^\top}.
#'
#' @export
#'
#' @examples
#' set.seed(123)
#' p <- 10; m <- 3
#' # True loading matrix (orthonormal columns)
#' A_true <- qr.Q(qr(matrix(rnorm(p * m), p, m)))
#' # Slightly perturbed estimate
#' A_est <- A_true + matrix(rnorm(p * m, sd = 0.05), p, m)
#' A_est <- qr.Q(qr(A_est))  # re-orthogonalize
#' 
#' calculate_DAA(A_est, A_true, m, p)
#'
calculate_DAA <- function(Ahat, A1, m, p) {
  epsilon <- 1e-6  
  A1tA1 <- A1 %*% t(A1) + epsilon * diag(nrow(A1))
  A1tA1_inv <- solve(A1tA1)
  result <- (1 - 1 / (m * (p)^2) * Re(t(Ahat) %*% A1tA1_inv %*% A1 %*% t(A1) %*% Ahat))^(1/2)
  return(result)
}

#' @name DIPC
#' @title Distributed Incremental Principal Component Analysis (DIPC)
#' @description Apply IPC in a distributed manner across K nodes.
#' @param data Matrix of input data (n × p).
#' @param m Number of principal components.
#' @param eta Proportion of  initial batch to  total data  within each node.
#' @param K Number of nodes (distributed splits).
#' @return List with per-node results and aggregated averages.
#' @examples
#' library(LaplacesDemon)
#' library(MASS)
#' n=1000
#' p=10
#' m=5
#' mu=t(matrix(rep(runif(p,0,1000),n),p,n))
#' mu0=as.matrix(runif(m,0))
#' sigma0=diag(runif(m,1))
#' F=matrix(mvrnorm(n,mu0,sigma0),nrow=n)
#' A=matrix(runif(p*m,-1,1),nrow=p)
#' lanor <- rlaplace(n*p,0,1)
#' epsilon=matrix(lanor,nrow=n)
#' D=diag(t(epsilon)%*%epsilon)
#' data=mu+F%*%t(A)+epsilon
#' results <- DIPC(data, m, eta=0.8, K=5)
#' @export
DIPC <- function(data, m, eta, K) {
  Ai_list <- list()
  Di_list <- list()
  n <- nrow(data)
  p <- ncol(data)
  chunk <- floor(n / K)
  if (chunk * K < n) warning("Rows not evenly divisible; trailing data ignored.")
  for (k in 1:K) {
    ## 1. 节点数据
    idx  <- ((k - 1) * chunk + 1):(k * chunk)
    Xk   <- scale(data[idx, ])             
    nk   <- nrow(Xk)
    n0   <- max(round(eta * nk), m)          
    Xbar <- colMeans(Xk[1:n0, ])
    S0   <- cov(Xk[1:n0, ])             
    eig  <- eigen(S0)
    lam  <- eig$values[1:m]
    V    <- eig$vectors[, 1:m, drop = FALSE] 
    
    ## 2. 在线更新
    for (i in (n0 + 1):nk) {
      xi <- Xk[i, ] - Xbar
      g  <- crossprod(V, xi)                 # V^T xi
      T  <- matrix(0, m + 1, m + 1)
      T[1:m, 1:m]       <- ((i - 1)/i) * diag(lam) + ((i - 1)^2/i^3) * tcrossprod(g)
      hmao <- norm(xi - V %*% g, "2")
      gamma <- as.numeric(crossprod(xi - V %*% g, xi)) / hmao
      T[1:m, m + 1] <- ((i - 1)^2/i^3) * gamma * g
      T[m + 1, ]    <- c(((i - 1)^2/i^3) * gamma * g,
                         ((i - 1)^2/i^3) * gamma^2)
      
      eigT <- eigen(T)
      lam  <- eigT$values[1:m]
      V    <- cbind(V, (xi - V %*% g)/hmao) %*% eigT$vectors[, 1:m, drop = FALSE]
      Xbar <- ((i - 1)/i) * Xbar + (1/i) * Xk[i, ]
    }
    
    # Store results for the i-th node
    Ai <- sqrt(lam) * V
    Di <- diag(cov(Xk) - tcrossprod(Ai))
    Ai_list[[k]] <- Ai
    Di_list[[k]]  <- Di
  }
  #Aggregate results across nodes (element-wise average)
  DAi <- Reduce(`+`, Ai_list) / K
  DDi <- Reduce(`+`, Di_list)  / K
  # Return per-node results and aggregated results
  return(list(node_results = list(Ai = Ai_list, Di = Di_list),
              aggregated   = list(DAi = DAi, DDi = DDi)))
}

#' @name FanPC
#' @title Apply the FanPC method to the Laplace factor model
#' @description This function performs Factor Analysis via Principal Component (FanPC) on a given data set. It calculates the estimated factor loading matrix (AF), specific variance matrix (DF), and the mean squared errors.
#' @param data A matrix of input data.
#' @param m is the number of principal component
#' @return AF,DF,SigmahatF
#' @examples
#' library(LaplacesDemon)
#' library(MASS)
#' n=1000
#' p=10
#' m=5
#' mu=t(matrix(rep(runif(p,0,1000),n),p,n))
#' mu0=as.matrix(runif(m,0))
#' sigma0=diag(runif(m,1))
#' F=matrix(mvrnorm(n,mu0,sigma0),nrow=n)
#' A=matrix(runif(p*m,-1,1),nrow=p)
#' lanor <- rlaplace(n*p,0,1)
#' epsilon=matrix(lanor,nrow=n)
#' D=diag(t(epsilon)%*%epsilon)
#' data=mu+F%*%t(A)+epsilon
#' results <- FanPC(data, m)
#' print(results)
#' @export
#' @importFrom matrixcalc frobenius.norm
#' @importFrom stats cor cov
FanPC=function(data,m){
  X=scale(data)
  n=nrow(X)
  SigmahatF=cor(X)
  eig<-eigen(SigmahatF)
  lambdahat = eig$values[1:m]
  ind<-order(lambdahat,decreasing=T)
  lambdahat<-lambdahat[ind]
  Q<- eig$vectors
  Q<-Q[,ind]
  AF<-Q[,1:m]
  hF <- diag(AF %*% t(AF))
  DF <- diag(SigmahatF - hF)
  return(
    list(AF=AF,DF=DF,SigmahatF=SigmahatF))}

#' The stochastic approximation principal component can handle online data sets with highly correlated.
#'
#' @param data is a highly correlated online data set
#' @param m is the number of principal component 
#' @param eta is the proportion of online data to total data
#'
#' @return Asa,Dsa
#' @export
#'
#' @examples
#' library(LaplacesDemon)
#' library(MASS)
#' n=1000
#' p=10
#' m=5
#' mu=t(matrix(rep(runif(p,0,1000),n),p,n))
#' mu0=as.matrix(runif(m,0))
#' sigma0=diag(runif(m,1))
#' F=matrix(mvrnorm(n,mu0,sigma0),nrow=n)
#' A=matrix(runif(p*m,-1,1),nrow=p)
#' lanor <- rlaplace(n*p,0,1)
#' epsilon=matrix(lanor,nrow=n)
#' D=diag(t(epsilon)%*%epsilon)
#' data=mu+F%*%t(A)+epsilon
#' SAPC(data=data,m=3,eta=0.8) 
SAPC<-function(data,m,eta){
  X<-as.matrix(scale(data))
  S<-cov(X)   
  n<-nrow(X)          
  n0<-round(eta*n)  
  p<-ncol(X)           
  Xbar<-colMeans(X[1:n0,])      
  eig1<-eigen(cov(X[1:n0,]-Xbar)) 
  lambda<-eig1$values[1:m]       
  V<-eig1$vectors[,1:m]      
  V1<-V
  
  for (i in (n0+1):n) {
    Xbar<-((i-1)/i)*Xbar+(1/i)*X[i,]
    Xcenter<-t(t(X[i,]-Xbar))
    gamma<-1/(i^(0.73))
    V<-V+gamma*Xcenter%*%t(Xcenter)%*%V
    V<-qr.Q(qr(V))
    lambda<- lambda+gamma*(t(V)%*%Xcenter%*%t(Xcenter)%*%V)
  }
  V2<-V
  Asa<-matrix(0,nrow=p,ncol=m)
  for (j in 1:m){
    Asa[,j]<-sqrt(lambda[j])*V2[,j]
  } 
  h2<-diag(Asa%*%t(Asa))
  Dsa<-diag(S-h2) 
  return(list(Asa=Asa,Dsa=Dsa))
}

#' @name frobenius.norm
#' @title Compute the Frobenius Norm of a Matrix
#'
#' @description Calculates the Frobenius norm of a matrix, defined as the square root of the sum of squares of all its entries. This is the standard Euclidean norm when the matrix is treated as a vector.
#'
#' @param M A numeric matrix.
#'
#' @return A non-negative scalar representing the Frobenius norm of \code{M}.
#'
#' @export
#'
#' @examples
#' M <- matrix(1:6, nrow = 2)
#' frobenius.norm(M)
#'
frobenius.norm <- function(M) {
  sqrt(sum(M^2))
}

#' The distributed stochastic approximation principal component for handling online data sets with highly correlated data across multiple nodes.
#'
#' @param data is a highly correlated online data set
#' @param m is the number of principal component 
#' @param eta is the proportion of online data to total data
#' @param n1 is the length of each data subset
#' @param K is the number of nodes
#'
#' @return Asa, Dsa (lists containing results from each node)
#' @export
#'
#' @examples
#' library(LaplacesDemon)
#' library(MASS)
#' n=1000
#' p=10
#' m=5
#' mu=t(matrix(rep(runif(p,0,1000),n),p,n))
#' mu0=as.matrix(runif(m,0))
#' sigma0=diag(runif(m,1))
#' F=matrix(mvrnorm(n,mu0,sigma0),nrow=n)
#' A=matrix(runif(p*m,-1,1),nrow=p)
#' lanor <- rlaplace(n*p,0,1)
#' epsilon=matrix(lanor,nrow=n)
#' D=diag(t(epsilon)%*%epsilon)
#' data=mu+F%*%t(A)+epsilon
#' DSAPC(data=data, m=3, eta=0.8, n1=128, K=2)
DSAPC <- function(data, m, eta, n1, K) {
  n <- nrow(data)
  p <- ncol(data)
  
  Asa_list <- list()
  Dsa_list <- list()
  
  for (i in 1:K) {
    L <- matrix(rep(0, K * n1), ncol = n1)
    R <- matrix(0, n1, n)
    L[i, ] <- sample(1:n, n1, replace = FALSE)
    r <- matrix(c(1:n1, L[i, ]), ncol = n1, byrow = TRUE)
    R[t(r)] <- 1
    
    X_subset <- R %*% as.matrix(data)
    
    sapc_result <- SAPC(data = X_subset, m = m, eta = eta)
    
    Asa_list[[i]] <- sapc_result$Asa
    Dsa_list[[i]] <- sapc_result$Dsa
  }
  
  return(list(Asa = Asa_list, Dsa = Dsa_list))
}

#' @name osdr_lfm
#' @title Online Sufficient Dimension Reduction for Laplace Factor Models (OSDR-LFM)
#'
#' @description Implements an online SIR-based sufficient dimension reduction method tailored for Laplace Factor Models (LFM) with symmetric, asymmetric, or skewed error structures.
#' Supports distributed deployment via local updates and global aggregation.
#'
#' @param X numeric matrix (n x p), observations in rows.
#' @param Y optional numeric vector (n) of proxy responses (e.g., factor scores). 
#'        If NULL, uses norm of projection as proxy (unsupervised LFM mode).
#' @param laplace_type character; one of "symmetric", "asymmetric", or "skewed".
#' @param K_max integer; maximum candidate dimension (default = min(10, p)).
#' @param H integer; number of slices for SIR (default = max(5, floor(sqrt(n)))).
#' @param method_svd character; "perturbation" or "gradient" (default = "gradient").
#' @param is_distributed logical; if TRUE, simulate distributed node behavior.
#' @param node_id integer; node identifier (only used if is_distributed = TRUE).
#' @param sync_interval integer; how often to "aggregate" in distributed mode (ignored if not distributed).
#' @param verbose logical; print progress.
#'
#' @return list with B_hat (p x K_est), K_est, lambda_trace, and (if distributed) local_B.
#'
#' @examples
#' set.seed(42)
#' n <- 600; p <- 30; m <- 4
#' A <- qr.Q(qr(matrix(rnorm(p * m), p, m)))
#' F <- matrix(rnorm(n * m), n, m)
#' eps <- matrix(rexp(n * p) - rexp(n * p), n, p)
#' X <- F %*% t(A) + eps
#' 
#' out <- osdr_lfm(X, laplace_type = "asymmetric", K_max = 6, verbose = TRUE)
#' cat("Estimated K:", out$K_est, "\n")
#' 
#' @importFrom stats quantile rnorm
#' @export
osdr_lfm <- function(X, Y = NULL, laplace_type = c("symmetric", "asymmetric", "skewed"),
                     K_max = NULL, H = NULL, method_svd = c("gradient", "perturbation"),
                     is_distributed = FALSE, node_id = 1, sync_interval = 50,
                     verbose = FALSE) {
  
  laplace_type <- match.arg(laplace_type)
  method_svd <- match.arg(method_svd)
  
  if (!is.matrix(X)) X <- as.matrix(X)
  n <- nrow(X); p <- ncol(X)
  
  if (is.null(K_max)) K_max <- min(10, p)
  if (K_max > p) K_max <- p
  if (is.null(H)) H <- max(5L, floor(sqrt(n)))
  
  # --- Step 0: Initialize ---
  B_t <- qr.Q(qr(matrix(rnorm(p * K_max), p, K_max)))  # p x K_max
  x_bar <- rep(0, p)
  Sigma_t <- diag(p) * 1e-6  # online covariance (regularized)
  M_t <- matrix(0, p, p)     # kernel matrix
  Gamma_t <- matrix(0, p, p) # running average of M (for perturbation method)
  
  # Parameters for asymmetric/skewed Laplace
  alpha_t <- rep(0, p)  # location shift (asymmetric)
  gamma_t <- rep(0, p)  # skewness (skewed)
  
  # Storage
  lambda_trace <- matrix(0, n, K_max)
  local_B_list <- list()
  
  # Slice boundaries (fixed in advance for stability)
  if (is.null(Y)) {
    # Unsupervised: use rolling projection norm as proxy
    Y_proxy <- numeric(n)
  } else {
    Y_proxy <- Y
  }
  
  for (t in 1:n) {
    x_new <- X[t, , drop = TRUE]
    
    # ---- Update mean and covariance (with Laplace adaptation) ----
    x_bar_old <- x_bar
    x_bar <- ((t - 1) / t) * x_bar + (1 / t) * x_new
    
    # Robust residual based on Laplace type
    resid <- x_new - x_bar
    if (laplace_type == "asymmetric") {
      # Estimate location shift alpha_t online
      alpha_t <- ((t - 1) / t) * alpha_t + (1 / t) * resid
      resid_adj <- resid - alpha_t
    } else if (laplace_type == "skewed") {
      # Simple skewness proxy: use sign-weighted magnitude
      skew_weight <- ifelse(resid > 0, 1.5, 0.5)
      gamma_t <- ((t - 1) / t) * gamma_t + (1 / t) * skew_weight * resid
      resid_adj <- resid - 2 * gamma_t
    } else {
      resid_adj <- resid
    }
    
    # Online covariance (rank-1 update)
    diff <- x_new - x_bar_old
    Sigma_t <- ((t - 1) / t) * Sigma_t + (1 / t) * tcrossprod(diff)
    
    # ---- Proxy response construction (if unsupervised) ----
    if (is.null(Y)) {
      proj_norm <- sqrt(sum((B_t %*% crossprod(B_t, x_new))^2))
      Y_proxy[t] <- proj_norm
    }
    y_t <- Y_proxy[t]
    
    # ---- Slice assignment (quantile-based for robustness) ----
    if (t < H) {
      slice_id <- 1
    } else {
      quantiles <- quantile(Y_proxy[1:t], probs = seq(0, 1, length.out = H + 1), na.rm = TRUE)
      slice_id <- findInterval(y_t, quantiles, all.inside = TRUE)
      slice_id <- max(1, min(H, slice_id))
    }
    
    # ---- Compute slice mean incrementally (simplified: single active slice) ----
    m_h <- solve(Sigma_t + diag(p) * 1e-6, resid_adj)  # approx \Sigma^{-1}(x - \bar{x})
    
    # Update kernel matrix: M_t = sum_h m_h m_h^T (approx with current h)
    M_t <- ((t - 1) / t) * M_t + (1 / t) * tcrossprod(m_h)
    
    # ---- Online SVD update ----
    if (method_svd == "gradient") {
      eta_t <- 1 / t
      grad <- M_t %*% B_t
      B_temp <- B_t + eta_t * grad
      B_t <- qr.Q(qr(B_temp))
    } else if (method_svd == "perturbation") {
      Gamma_t <- ((t - 1) / t) * Gamma_t + (1 / t) * M_t
      # Perturbation requires eigen-decomp at each step (costly for large p)
      eig_M <- eigen(M_t, symmetric = TRUE)
      lambda_t <- eig_M$values[1:K_max]
      beta_t <- eig_M$vectors[, 1:K_max, drop = FALSE]
      B_t <- beta_t
      Gamma_t <- M_t  # simplify: treat Gamma_t = M_t
    }
    
    # ---- Online eigenvalue trace (for BIC) ----
    Mt_proj <- crossprod(B_t, M_t %*% B_t)
    eig_vals <- eigen(Mt_proj, symmetric = TRUE, only.values = TRUE)$values
    eig_vals <- pmax(eig_vals, 0)
    lambda_trace[t, ] <- eig_vals
    
    # ---- Distributed simulation: store local B periodically ----
    if (is_distributed && (t %% sync_interval == 0 || t == n)) {
      local_B_list[[length(local_B_list) + 1]] <- B_t
    }
    
    if (verbose && (t %% 100 == 0 || t == n)) {
      cat(sprintf("Step %d | Slice: %d | Laplace: %s\n", t, slice_id, laplace_type))
    }
  }
  
  # ---- Final dimension selection (BIC-like) ----
  total_var <- rowSums(lambda_trace^2)
  D_vals <- matrix(0, n, K_max)
  for (k in 1:K_max) {
    ratio <- rowSums(lambda_trace[, 1:k, drop = FALSE]^2) / pmax(total_var, 1e-12)
    penalty <- (sqrt(1:n) * k * (k + 1)) / (2 * (1:n))
    D_vals[, k] <- ratio - penalty
  }
  K_est <- which.max(D_vals[n, ])
  B_final <- B_t[, 1:K_est, drop = FALSE]
  
  # Ensure orthonormality
  B_final <- qr.Q(qr(B_final))
  
  result <- list(
    B_hat = B_final,
    K_est = K_est,
    lambda_trace = lambda_trace,
    laplace_type = laplace_type
  )
  
  if (is_distributed) {
    result$local_B_history <- local_B_list
    result$node_id <- node_id
  }
  
  return(result)
}

#' @name generate_asymmetric_laplace
#' @title Generate Multivariate Asymmetric Laplace Random Vectors
#'
#' @description Generates a matrix of independent draws from a multivariate asymmetric Laplace distribution. Each row is \eqn{\epsilon = \alpha W + Z\sqrt{W}}, where \eqn{W \sim \text{Exp}(1)} independent of \eqn{Z \sim N_p(0, \Sigma)}. The parameter \eqn{\alpha} controls the asymmetry (skewness) in each coordinate.
#'
#' @param n Number of observations (rows) to generate.
#' @param p Dimension of each observation (columns).
#' @param alpha A numeric vector of length \code{p} giving the asymmetry (shift) parameters for each coordinate. Can be a scalar if \code{p = 1}.
#' @param Sigma A \eqn{p \times p} positive definite covariance matrix for the Gaussian component \eqn{Z}.
#'
#' @return An \code{n x p} matrix where each row is a draw from the multivariate asymmetric Laplace distribution with shift vector \code{alpha} and scale matrix \code{Sigma}.
#'
#' @importFrom MASS mvrnorm
#' @export
#'
#' @examples
#' set.seed(123)
#' p <- 3
#' alpha <- c(0.5, -0.3, 0.2)
#' Sigma <- diag(p)
#' X <- generate_asymmetric_laplace(100, p, alpha, Sigma)
#' pairs(X)
#' @importFrom stats rexp
generate_asymmetric_laplace <- function(n, p, alpha, Sigma) {
  W <- rexp(n, rate = 1)
  Z <- MASS::mvrnorm(n, mu = rep(0, p), Sigma = Sigma)
  epsilon <- matrix(rep(alpha, each = n), nrow = n) * W + Z * sqrt(W)
  return(epsilon)
}  

#' Distributed Factor Model Testing with Wald, GRS, PY tests and FDR control
#'
#' Performs comprehensive factor model testing in distributed environment across multiple nodes,
#' including joint tests (Wald, GRS, PY), individual asset t-tests, and False Discovery Rate control.
#'
#' @param ret A T × N matrix representing the excess returns of N assets at T time points.
#' @param fac A T × K matrix representing the returns of K factors at T time points.
#' @param n1 The number of assets allocated to each node
#' @param K The number of nodes
#' @param q.fdr The significance level for FDR (False Discovery Rate) testing, defaulting to 5\%.
#'
#' @return A list containing the following components:
#' \item{alpha_list}{List of alpha vectors from each node}
#' \item{tstat_list}{List of t-statistics from each node}
#' \item{pval_list}{List of p-values from each node}
#' \item{Wald_list}{List of Wald test statistics from each node}
#' \item{p_Wald_list}{List of p-values for Wald tests from each node}
#' \item{GRS_list}{List of GRS test statistics from each node}
#' \item{p_GRS_list}{List of p-values for GRS tests from each node}
#' \item{PY_list}{List of Pesaran and Yamagata test statistics from each node}
#' \item{p_PY_list}{List of p-values for PY tests from each node}
#' \item{reject_fdr_list}{List of logical vectors indicating significant assets after FDR correction from each node}
#' \item{power_proxy_list}{List of number of significant assets after FDR correction from each node}
#' \item{combined_alpha}{Combined alpha vector from all nodes}
#' \item{combined_pval}{Combined p-value vector from all nodes}
#' \item{combined_reject_fdr}{Combined FDR rejection vector from all nodes}
#' \item{total_power_proxy}{Total number of significant assets across all nodes after FDR correction}
#'
#' @examples
#' set.seed(42)
#' T <- 120
#' N <- 100  # Larger dataset for distributed testing
#' K_factors <- 3
#' fac <- matrix(rnorm(T * K_factors), T, K_factors)
#' beta <- matrix(rnorm(N * K_factors), N, K_factors)
#' alpha <- rep(0, N)
#' alpha[1:10] <- 0.4 / 100  # 10 non-zero alphas
#' eps <- matrix(rnorm(T * N, sd = 0.02), T, N)
#' ret <- alpha + fac %*% t(beta) + eps
#' 
#' # Distributed testing with 4 nodes, each handling 25 assets
#' results <- Dfactor.tests(ret, fac, n1 = 25, K = 4, q.fdr = 0.05)
#' 
#' # View combined results
#' cat("Total significant assets after FDR:", results$total_power_proxy, "\n")
#' cat("Combined results across all nodes:\n")
#' print(summary(results$combined_alpha))
#'
#' @export
#' @importFrom stats lm residuals coef cov cor pchisq pf pnorm p.adjust
#' @importFrom MASS ginv
Dfactor.tests <- function(ret, fac, n1, K, q.fdr = 0.05) {
  T <- nrow(ret)
  N <- ncol(ret)
  K_factors <- ncol(fac)
  
  # Validate input parameters
  if (n1 * K > N) {
    stop("n1 * K cannot exceed total number of assets N")
  }
  
  # Initialize lists to store results from each node
  alpha_list <- list()
  tstat_list <- list()
  pval_list <- list()
  Wald_list <- list()
  p_Wald_list <- list()
  GRS_list <- list()
  p_GRS_list <- list()
  PY_list <- list()
  p_PY_list <- list()
  reject_fdr_list <- list()
  power_proxy_list <- list()
  
  # Create asset allocation matrix for distributed processing
  L <- matrix(rep(0, K * n1), ncol = n1)
  R <- matrix(0, n1, N)
  
  for (i in 1:K) {
    # Sample assets for current node
    L[i, ] <- sample(1:N, n1, replace = FALSE)
    r <- matrix(c(1:n1, L[i, ]), ncol = n1, byrow = TRUE)
    R[t(r)] <- 1
    
    # Extract asset returns for current node
    ret_node <- ret[, L[i, ]]
    
    # Perform factor model testing on node subset
    node_results <- .node.factor.tests(ret_node, fac, q.fdr)
    
    # Store node results
    alpha_list[[i]] <- node_results$alpha
    tstat_list[[i]] <- node_results$tstat
    pval_list[[i]] <- node_results$pval
    Wald_list[[i]] <- node_results$Wald
    p_Wald_list[[i]] <- node_results$p_Wald
    GRS_list[[i]] <- node_results$GRS
    p_GRS_list[[i]] <- node_results$p_GRS
    PY_list[[i]] <- node_results$PY
    p_PY_list[[i]] <- node_results$p_PY
    reject_fdr_list[[i]] <- node_results$reject_fdr
    power_proxy_list[[i]] <- node_results$power_proxy
  }
  
  # Combine results across all nodes
  combined_results <- .combine.node.results(alpha_list, pval_list, reject_fdr_list, 
                                            power_proxy_list, L, N, q.fdr)
  
  # Return comprehensive results
  return(list(
    alpha_list = alpha_list,
    tstat_list = tstat_list,
    pval_list = pval_list,
    Wald_list = Wald_list,
    p_Wald_list = p_Wald_list,
    GRS_list = GRS_list,
    p_GRS_list = p_GRS_list,
    PY_list = PY_list,
    p_PY_list = p_PY_list,
    reject_fdr_list = reject_fdr_list,
    power_proxy_list = power_proxy_list,
    combined_alpha = combined_results$combined_alpha,
    combined_pval = combined_results$combined_pval,
    combined_reject_fdr = combined_results$combined_reject_fdr,
    total_power_proxy = combined_results$total_power_proxy,
    asset_allocation = L
  ))
}

#' Helper function for node-level factor model testing
#' @noRd
.node.factor.tests <- function(ret_node, fac, q.fdr) {
  T <- nrow(ret_node)
  N_node <- ncol(ret_node)
  K <- ncol(fac)
  
  # Asset-by-asset OLS
  alpha <- numeric(N_node)
  tstat <- numeric(N_node)
  pval <- numeric(N_node)
  resid <- matrix(NA, T, N_node)
  
  for (j in 1:N_node) {
    mod <- lm(ret_node[, j] ~ fac)
    alpha[j] <- coef(mod)[1]
    resid[, j] <- residuals(mod)
    tstat[j] <- summary(mod)$coefficients[1, 3]
    pval[j] <- summary(mod)$coefficients[1, 4]
  }
  
  # Residual covariance matrices
  Sigma_u <- cov(resid)
  Sigma_u_inv <- tryCatch(solve(Sigma_u),
                          error = function(e) MASS::ginv(Sigma_u))
  D_hat <- diag(diag(Sigma_u))
  
  # Joint tests
  # Wald test
  Wald <- T * as.numeric(t(alpha) %*% Sigma_u_inv %*% alpha)
  p_Wald <- pchisq(Wald, df = N_node, lower.tail = FALSE)
  
  # GRS test
  dof_num <- N_node
  dof_den <- T - N_node - K
  if (dof_den <= 0) {
    GRS <- p_GRS <- NA_real_
  } else {
    fbar <- colMeans(fac)
    Omega <- cov(fac)
    adj <- 1 + as.numeric(t(fbar) %*% solve(Omega) %*% fbar)
    GRS <- (dof_den / dof_num) * (Wald / adj) / (T - K - 1)
    p_GRS <- pf(GRS, dof_num, dof_den, lower.tail = FALSE)
  }
  
  # PY test
  kappa <- 1 + (2/N_node) * sum(cor(resid)^2)
  PY_num <- as.numeric(t(alpha) %*% solve(D_hat) %*% alpha) - N_node
  PY <- PY_num / sqrt(2 * N_node * kappa)
  p_PY <- pnorm(PY, lower.tail = FALSE)
  
  # FDR on asset-level p-values
  bh <- p.adjust(pval, method = "BH")
  reject_fdr <- bh <= q.fdr
  power_proxy <- sum(reject_fdr)
  
  return(list(
    alpha = alpha,
    tstat = tstat,
    pval = pval,
    Wald = Wald,
    p_Wald = p_Wald,
    GRS = GRS,
    p_GRS = p_GRS,
    PY = PY,
    p_PY = p_PY,
    reject_fdr = reject_fdr,
    power_proxy = power_proxy
  ))
}

#' Helper function to combine results from all nodes
#' @noRd
.combine.node.results <- function(alpha_list, pval_list, reject_fdr_list, 
                                  power_proxy_list, L, N, q.fdr) {
  K <- length(alpha_list)
  
  # Initialize combined vectors
  combined_alpha <- rep(NA, N)
  combined_pval <- rep(NA, N)
  combined_reject_fdr <- rep(FALSE, N)
  
  # Combine results from all nodes
  for (i in 1:K) {
    assets <- L[i, ]
    combined_alpha[assets] <- alpha_list[[i]]
    combined_pval[assets] <- pval_list[[i]]
    combined_reject_fdr[assets] <- reject_fdr_list[[i]]
  }
  
  # Apply global FDR correction across all assets
  global_bh <- p.adjust(combined_pval, method = "BH")
  global_reject_fdr <- global_bh <= q.fdr
  total_power_proxy <- sum(global_reject_fdr)
  
  return(list(
    combined_alpha = combined_alpha,
    combined_pval = combined_pval,
    combined_reject_fdr = global_reject_fdr,
    total_power_proxy = total_power_proxy
  ))
}

#' @name generate_skew_laplace
#' @title Generate Multivariate Skew Laplace Random Vectors
#'
#' @description Generates a matrix of independent draws from a multivariate skew Laplace distribution with location vector \eqn{\alpha}, scale matrix \eqn{\Sigma}, and skewness parameter \eqn{\gamma}. Each observation is constructed as \eqn{\epsilon = \alpha + \gamma V + \sqrt{V} Z}, where \eqn{V \sim \text{Gamma}((p+1)/2, 1/2)} and \eqn{Z \sim N_p(0, \Sigma)} independent of \eqn{V}.
#'
#' @param n Number of observations (rows) to generate.
#' @param p Dimension of each observation (columns).
#' @param alpha Numeric vector of length \code{p} giving the location (shift) parameter. Can be a scalar if \code{p = 1}.
#' @param gamma Numeric vector of length \code{p} giving the skewness parameter for each coordinate. Can be a scalar if \code{p = 1}.
#' @param Sigma A \eqn{p \times p} positive definite covariance matrix for the Gaussian component \eqn{Z}.
#'
#' @return An \code{n x p} matrix where each row is a draw from the specified multivariate skew Laplace distribution.
#'
#' @importFrom MASS mvrnorm
#' @importFrom stats rgamma
#' @export
#'
#' @examples
#' set.seed(123)
#' p <- 3
#' alpha <- c(0.1, -0.2, 0.0)
#' gamma <- c(0.5, 0.3, -0.4)
#' Sigma <- diag(p)
#' X <- generate_skew_laplace(100, p, alpha, gamma, Sigma)
#' pairs(X)
#'
generate_skew_laplace <- function(n, p, alpha, gamma, Sigma) {
  epsilon <- matrix(0, nrow = n, ncol = p)
  shape_param <- (p + 1)/2
  for (i in 1:n) {
    V <- rgamma(1, shape = shape_param, rate = 0.5)
    Z <- MASS::mvrnorm(1, mu = rep(0, p), Sigma = Sigma)
    epsilon[i, ] <- alpha + gamma * V + sqrt(V) * Z
  }
  return(epsilon)
}

#' @name GulPC
#' @title General unilateral loading principal component
#' @param data is a total data set
#' @param m is the number of first layer principal component
#' @return AU1,AU2,DU3,SigmaUhat
#' @export
#' @examples
#' library(LaplacesDemon)
#' library(MASS)
#' n=1000
#' p=10
#' m=5
#' mu=t(matrix(rep(runif(p,0,1000),n),p,n))
#' mu0=as.matrix(runif(m,0))
#' sigma0=diag(runif(m,1))
#' F=matrix(mvrnorm(n,mu0,sigma0),nrow=n)
#' A=matrix(runif(p*m,-1,1),nrow=p)
#' lanor <- rlaplace(n*p,0,1)
#' epsilon=matrix(lanor,nrow=n)
#' D=diag(t(epsilon)%*%epsilon)
#' data=mu+F%*%t(A)+epsilon
#' GulPC(data=data,m=5)
#' @importFrom stats cor cov
GulPC=function(data,m){
  X=scale(data)
  n=nrow(X)
  p<-ncol(X)
  SigmaU1hat=cor(X)
  eig5<-eigen(SigmaU1hat)
  lambda1hat = eig5$values[1:m]
  ind<-order(lambda1hat,decreasing=T)
  lambda1hat<-lambda1hat[ind]
  Q1<-eig5$vectors
  Q1=Q1[,ind]
  Q1hat<- Q1[, 1:m]
  AU1 <- matrix(0, nrow = p, ncol = m)
  for (j in 1:m) {AU1[, j] <- sqrt(lambda1hat[j]) * Q1hat[, j]}; AU1
  hU1 <- diag(AU1 %*% t(AU1))
  DU1 <- diag(SigmaU1hat - hU1)
  pc=2
  F1hat=X%*%AU1
  F1star<-F1hat/sqrt(n)
  SigmaU2hat=cov(F1star)
  eig6<-eigen(SigmaU2hat)
  lambda2hat =eig6$values[1:pc]
  ind<-order(lambda2hat,decreasing=T)
  lambda2hat<-lambda2hat[ind]
  Q2<-eig6$vectors
  Q2=Q2[,ind]
  Q2hat<- Q2[, 1:pc]
  AU2<- matrix(0, nrow = m, ncol = pc)
  for (j in 1:pc) {AU2[, j] <- sqrt(lambda2hat[j]) * Q2hat[, j]}; AU2
  hU2 <- diag(AU2%*% t(AU2))
  DU2 <- diag(SigmaU2hat - hU2)
  Fhat=F1star%*%AU2
  Xhat=Fhat%*%t(AU2)%*%t(AU1)
  S1hat=cov(Xhat)
  hU3 <- diag(t(t(AU2)%*%t(AU1))%*%(t(AU2)%*%t(AU1)))
  DU3 <- diag(S1hat - hU3)
  return(list(AU1=AU1,AU2=AU2,DU3=DU3,S1hat=S1hat))}

#' @name estimate_AD_from_OSDR
#' @title Estimate Factor Loadings and Uniquenesses from OSDR Output
#'
#' @description Extracts the estimated factor loading matrix \eqn{A} and the diagonal noise covariance matrix \eqn{D} from the output of \code{\link{online_sir_lfm}}. The subspace estimate \code{B_hat} is truncated or padded to have exactly \code{m} columns, used as \eqn{A}, and \eqn{D} is obtained by subtracting the factor covariance contribution from the sample covariance matrix, with a lower bound on the diagonal entries to ensure positivity.
#'
#' @param data The original data matrix (n x p) used in the online SIR procedure.
#' @param res A list returned by \code{\link{online_sir_lfm}}, containing at least the component \code{B_hat} (p x K_est matrix).
#' @param m The target number of factors (structural dimension). If \code{B_hat} has fewer columns, the missing directions are filled with zeros; if more, only the first \code{m} columns are retained.
#'
#' @return A list with components:
#' \item{A_est}{Estimated factor loading matrix of dimension p x m.}
#' \item{D_est}{Estimated diagonal noise covariance matrix of dimension p x p, with diagonal entries forced to be at least 0.01.}
#'
#' @export
#'
#' @importFrom stats cov
#'
#' @examples
#' \dontrun{
#' set.seed(123)
#' n <- 500; p <- 20; m <- 3
#' B_true <- qr.Q(qr(matrix(rnorm(p * m), p, m)))
#' f <- matrix(rnorm(n * m), n, m)
#' eps <- matrix(rexp(n * p, rate = 1) - 1, n, p) # Laplace noise
#' X <- f %*% t(B_true) + eps
#'
#' # Run online SIR
#' res <- online_sir_lfm(X, K_true = m, method = "gradient", verbose = FALSE)
#'
#' # Estimate A and D from the result
#' AD <- estimate_AD_from_OSDR(X, res, m)
#' print(head(AD$A_est))
#' print(diag(AD$D_est)[1:5])
#' }
#'
estimate_AD_from_OSDR <- function(data, res, m) {
  p <- ncol(data)
  B_hat <- res$B_hat  
  if (ncol(B_hat) < m) {
    B_hat <- cbind(B_hat, matrix(0, nrow = p, ncol = m - ncol(B_hat)))
  } else if (ncol(B_hat) > m) {
    B_hat <- B_hat[, 1:m, drop = FALSE]
  }
  A_est <- B_hat
  S_sample <- cov(data)
  S_factors <- A_est %*% t(A_est)
  D_est <- diag(diag(S_sample - S_factors))
  diag(D_est) <- pmax(diag(D_est), 0.01)
  return(list(A_est = A_est, D_est = D_est))
}

#' @name DGulPC
#' @title Distributed general unilateral loading principal component
#' @param data is a total data set
#' @param m is the number of principal component
#' @param n1 is  the length of each data subset
#' @param K is the number of nodes
#' @return AU1,AU2,DU3,Shat
#' @export
#' @examples
#' library(LaplacesDemon)
#' library(MASS)
#' n=1000
#' p=10
#' m=5
#' mu=t(matrix(rep(runif(p,0,1000),n),p,n))
#' mu0=as.matrix(runif(m,0))
#' sigma0=diag(runif(m,1))
#' F=matrix(mvrnorm(n,mu0,sigma0),nrow=n)
#' A=matrix(runif(p*m,-1,1),nrow=p)
#' lanor <- rlaplace(n*p,0,1)
#' epsilon=matrix(lanor,nrow=n)
#' D=diag(t(epsilon)%*%epsilon)
#' data=mu+F%*%t(A)+epsilon
#' DGulPC(data,m=3,n1=128,K=2)
#' @importFrom stats cor cov
DGulPC=function(data,m,n1,K){
  SigmaU1hat=list()
  SigmaU2hat=list()
  AU1=list()
  AU2=list()
  DU3=list()
  shat=list()
  Fhat=list()
  F1hat=list()
  DU1=list()
  DU2=list()
  for (i in 1:K) {
    n=nrow(data)
    p=ncol(data)
    pc=2
    L=matrix(rep(0,K*n1),ncol=n1)
    R=matrix(0,n1,n)
    L[i,]=sample(1:n,n1,replace=FALSE)
    r=matrix(c(1:n1,L[i,]),ncol=n1,byrow=T)
    R[t(r)]=1
    X1=R%*%as.matrix(data)
    X=scale(X1)
    SigmaU1hat[[i]]<-cor(X)
    eig5<-eigen(SigmaU1hat[[i]])
    lambda1hat = eig5$values[1:m]
    ind<-order(lambda1hat,decreasing=T)
    lambda1hat<-lambda1hat[ind]
    Q1<-eig5$vectors
    Q1=Q1[,ind]
    Q1hat<- Q1[, 1:m]
    AU11 <- matrix(0, nrow = p, ncol = m)
    for (j in 1:m) {AU11[, j] <- sqrt(lambda1hat[j]) * Q1hat[, j]}
    AU1[[i]]=AU11
    hU1 <- diag(AU1[[i]] %*% t(AU1[[i]]))
    DU1[[i]] <- diag(SigmaU1hat[[i]] - hU1)
    p<-ncol(X)
    F1hat[[i]]=X%*%AU1[[i]]
    F1star<-F1hat[[i]]/sqrt(n)
    SigmaU2hat[[i]]=cor(F1star)
    eig6<-eigen(SigmaU2hat[[i]])
    lambda2hat =eig6$values[1:pc]
    ind<-order(lambda2hat,decreasing=T)
    lambda2hat<-lambda2hat[ind]
    Q2<-eig6$vectors
    Q2=Q2[,ind]
    Q2hat<- Q2[, 1:pc]
    AU22<- matrix(0, nrow = m, ncol = pc)
    for (j in 1:pc) {AU22[, j] <- sqrt(lambda2hat[j]) * Q2hat[, j]}
    AU2[[i]]=AU22
    hU2 <- diag(AU2[[i]]%*% t(AU2[[i]]))
    DU2[[i]] <- diag(SigmaU2hat[[i]] - hU2)
    Fhat[[i]]=F1star%*%AU2[[i]]
    Xhat=Fhat[[i]]%*%t(AU2[[i]])%*%t(AU1[[i]])
    shat[[i]]=cov(Xhat)
    hU3 <- diag(t(t(AU2[[i]])%*%t(AU1[[i]]))%*%(t(AU2[[i]])%*%t(AU1[[i]])))
    DU3[[i]] <- diag(shat[[i]] - hU3)
  return(list(AU1=AU1,AU2=AU2,DU3=DU3,shat=shat))}}

#' Factor Model Testing with Wald, GRS, PY tests and FDR control
#'
#' Performs comprehensive factor model testing including joint tests (Wald, GRS, PY), 
#' individual asset t-tests, and False Discovery Rate control.
#'
#' @param ret A T × N matrix representing the excess returns of N assets at T time points.
#' @param fac A T × K matrix representing the returns of K factors at T time points.
#' @param q.fdr The significance level for FDR (False Discovery Rate) testing, defaulting to 5\%.
#'
#' @return A list containing the following components:
#' \item{alpha}{N-vector of estimated alphas for each asset}
#' \item{tstat}{N-vector of t-statistics for testing individual alphas}
#' \item{pval}{N-vector of p-values for individual alpha tests}
#' \item{Wald}{Wald test statistic for joint alpha significance}
#' \item{p_Wald}{p-value for Wald test}
#' \item{GRS}{GRS test statistic (finite-sample F-test)}
#' \item{p_GRS}{p-value for GRS test}
#' \item{PY}{Pesaran and Yamagata test statistic}
#' \item{p_PY}{p-value for PY test}
#' \item{reject_fdr}{Logical vector indicating which assets have significant alphas after FDR correction}
#' \item{fdr_p}{Adjusted p-values using Benjamini-Hochberg procedure}
#' \item{power_proxy}{Number of significant assets after FDR correction}
#'
#' @examples
#' set.seed(42)
#' T <- 120
#' N <- 25
#' K <- 3
#' fac <- matrix(rnorm(T * K), T, K)
#' beta <- matrix(rnorm(N * K), N, K)
#' alpha <- rep(0, N)
#' alpha[1:3] <- 0.4 / 100  # 3 non-zero alphas
#' eps <- matrix(rnorm(T * N, sd = 0.02), T, N)
#' ret <- alpha + fac %*% t(beta) + eps
#' results <- factor.tests(ret, fac, q.fdr = 0.05)
#' 
#' # View results
#' cat("Wald test p-value:", results$p_Wald, "\n")
#' cat("GRS test p-value:", results$p_GRS, "\n")
#' cat("PY test p-value:", results$p_PY, "\n")
#' cat("Significant assets after FDR:", results$power_proxy, "\n")
#'
#' @export
#' @importFrom stats lm residuals coef cov cor pchisq pf pnorm p.adjust
#' @importFrom MASS ginv
factor.tests <- function(ret, fac, q.fdr = 0.05) {
  T  <- nrow(ret)
  N  <- ncol(ret)
  K  <- ncol(fac)
  
  # --- 1. asset-by-asset OLS ---
  alpha <- numeric(N)
  tstat <- numeric(N)
  pval  <- numeric(N)
  resid <- matrix(NA, T, N)
  
  for (j in 1:N) {
    mod <- lm(ret[, j] ~ fac)
    alpha[j] <- coef(mod)[1]
    resid[, j] <- residuals(mod)
    tstat[j]  <- summary(mod)$coefficients[1, 3]
    pval[j]   <- summary(mod)$coefficients[1, 4]
  }
  
  # --- 2. residual covariance matrices ---
  Sigma_u <- cov(resid)          # full
  Sigma_u_inv <- tryCatch(solve(Sigma_u),
                          error = function(e) MASS::ginv(Sigma_u))
  D_hat   <- diag(diag(Sigma_u)) # diagonal only (PY)
  
  # --- 3. joint tests ---
  # 3a. Wald
  Wald <- T * as.numeric(t(alpha) %*% Sigma_u_inv %*% alpha)
  p_Wald <- pchisq(Wald, df = N, lower.tail = FALSE)
  
  # 3b. GRS (finite-sample F)
  dof_num <- N
  dof_den <- T - N - K
  if (dof_den <= 0) {
    GRS <- p_GRS <- NA_real_
  } else {
    fbar  <- colMeans(fac)
    Omega <- cov(fac)
    adj   <- 1 + as.numeric(t(fbar) %*% solve(Omega) %*% fbar)
    GRS   <- (dof_den / dof_num) * (Wald / adj) / (T - K - 1)
    p_GRS <- pf(GRS, dof_num, dof_den, lower.tail = FALSE)
  }
  
  # 3c. PY (working-independent)
  kappa <- 1 + (2/N) * sum(cor(resid)^2)
  PY_num <- as.numeric(t(alpha) %*% solve(D_hat) %*% alpha) - N
  PY <- PY_num / sqrt(2 * N * kappa)
  p_PY <- pnorm(PY, lower.tail = FALSE)
  
  # --- 4. FDR on asset-level p-values ---
  bh <- p.adjust(pval, method = "BH")
  reject_fdr <- bh <= q.fdr
  power_proxy <- sum(reject_fdr)
  
  # --- 5. return everything ---
  list(alpha = alpha,
       tstat = tstat,
       pval  = pval,
       Wald  = Wald,  p_Wald = p_Wald,
       GRS   = GRS,   p_GRS  = p_GRS,
       PY    = PY,    p_PY   = p_PY,
       reject_fdr = reject_fdr,
       fdr_p      = bh,
       power_proxy= power_proxy)
}

#' @name generate_multivariate_laplace
#' @title Generate Multivariate Symmetric Laplace Random Vectors
#'
#' @description Generates a matrix of independent draws from a multivariate symmetric Laplace distribution. Each row is an independent observation of a \eqn{p}-dimensional random vector \eqn{\epsilon = Z\sqrt{W}}, where \eqn{Z \sim N_p(0, \Sigma_L)} and \eqn{W \sim \text{Exp}(1)} independent of \eqn{Z}.
#'
#' @param n Number of observations (rows) to generate.
#' @param p Dimension of each observation (columns).
#' @param Sigma_L A \eqn{p \times p} positive definite covariance matrix for the Gaussian component \eqn{Z}.
#'
#' @return An \code{n x p} matrix where each row is a draw from the multivariate symmetric Laplace distribution with scale matrix \eqn{\Sigma_L}.
#'
#' @importFrom MASS mvrnorm
#' @export
#'
#' @examples
#' set.seed(123)
#' p <- 3
#' Sigma_L <- diag(p)
#' X <- generate_multivariate_laplace(100, p, Sigma_L)
#' pairs(X)
#' @importFrom stats rexp
generate_multivariate_laplace <- function(n, p, Sigma_L) {
  W <- rexp(n, rate = 1)
  Z <- MASS::mvrnorm(n, mu = rep(0, p), Sigma = Sigma_L)
  epsilon <- Z * sqrt(W)
  return(epsilon)
}

#' @name DPPC
#' @title Distributed projection principal component
#' @param data is a total data set
#' @param m is the number of principal component
#' @param n1 is  the length of each data subset
#' @param K is the number of nodes
#' @return Apro,pro,Sigmahathatpro
#' @export
#' @examples
#' library(LaplacesDemon)
#' library(MASS)
#' n=1000
#' p=10
#' m=5
#' mu=t(matrix(rep(runif(p,0,1000),n),p,n))
#' mu0=as.matrix(runif(m,0))
#' sigma0=diag(runif(m,1))
#' F=matrix(mvrnorm(n,mu0,sigma0),nrow=n)
#' A=matrix(runif(p*m,-1,1),nrow=p)
#' lanor <- rlaplace(n*p,0,1)
#' epsilon=matrix(lanor,nrow=n)
#' D=diag(t(epsilon)%*%epsilon)
#' data=mu+F%*%t(A)+epsilon
#' DPPC(data,m=3,n1=128,K=2)
#' @importFrom stats cor cov
DPPC=function(data,m,n1,K){
  n=nrow(data)
  p=ncol(data)
  P=as.matrix(diag(c(0,1),n,n))
  X1pro=scale(P%*%as.matrix(data))
  Sigmahatpro=list()
  Apro=list()
  Dpro=list()
  for (i in 1:K) {
    L=matrix(rep(0,K*n1),ncol=n1)
    R=matrix(0,n1,n)
    L[i,]=sample(1:n,n1,replace=FALSE)
    r=matrix(c(1:n1,L[i,]),ncol=n1,byrow=T)
    R[t(r)]=1
    X1=R%*%X1pro
    Xpro=scale(X1)
    Sigmahatpro[[i]]<-cor(Xpro)
    eig1<-eigen(Sigmahatpro[[i]])
    lambdahat =eig1$values[1:m]
    ind<-order(lambdahat,decreasing=T)
    lambdahat<-lambdahat[ind]
    Q <- eig1$vectors
    Q<-Q[,ind]
    Qhat<-Q[,1:m]
    Apro1 <- matrix(0, nrow = p, ncol = m)
    for (j in 1:m) {Apro1[, j] <- sqrt(lambdahat[j]) * Qhat[, j]}
    Apro[[i]] =Apro1
    hpro <- diag(Apro[[i]] %*% t(Apro[[i]]))
    Dpro[[i]] <- diag(Sigmahatpro[[i]] - hpro) }
  return(list(Apro=Apro,Dpro=Dpro,Sigmahatpro=Sigmahatpro))}

#' @name PPC
#' @title Projection principal component 
#' @param data is a total  data set
#' @param m is the number of principal component
#' @return Apro, Dpro, Sigmahatpro
#' @export
#' @examples
#' library(LaplacesDemon)
#' library(MASS)
#' n=1000
#' p=10
#' m=5
#' mu=t(matrix(rep(runif(p,0,1000),n),p,n))
#' mu0=as.matrix(runif(m,0))
#' sigma0=diag(runif(m,1))
#' F=matrix(mvrnorm(n,mu0,sigma0),nrow=n)
#' A=matrix(runif(p*m,-1,1),nrow=p)
#' lanor <- rlaplace(n*p,0,1)
#' epsilon=matrix(lanor,nrow=n)
#' D=diag(t(epsilon)%*%epsilon)
#' data=mu+F%*%t(A)+epsilon
#' PPC(data=data,m=5)
#' @importFrom stats cor cov
PPC=function(data,m){
  X=scale(data)
  n=nrow(X)
  p=ncol(X)
  P=as.matrix(diag(c(0,1),n,n))
  Xpro=scale(P%*%X)
  Sigmahatpro<-cov(Xpro)
  eig<-eigen(Sigmahatpro)
  lambdahat =eig$values[1:m]
  ind<-order(lambdahat,decreasing=T)
  lambdahat<-lambdahat[ind]
  Q <- eig$vectors
  Q<-Q[,ind]
  Qhat<-Q[,1:m]
  Apro <- matrix(0, nrow = p, ncol = m)
  for (j in 1:m) {Apro[, j] <- sqrt(lambdahat[j]) * Qhat[, j]}; Apro
  hpro <- diag(Apro %*% t(Apro))
  Dpro <- diag(Sigmahatpro - hpro)
  return(list(Apro=Apro,Dpro=Dpro,Sigmahatpro=Sigmahatpro))}

#' @name LFM
#' @title Generate Laplace factor models
#' @description The function is to generate Laplace factor model data.
#' The function supports various distribution types for generating the data,
#' including:
#' - `truncated_laplace`: Truncated Laplace distribution
#' - `log_laplace`: Univariate Symmetric Log-Laplace distribution
#' - `Asymmetric Log_Laplace`: Log-Laplace distribution
#' - `Skew-Laplace`: Skew-Laplace distribution
#' @usage LFM(n, p, m, distribution_type)
#' @param n An integer specifying the sample size.
#' @param p An integer specifying the sample dimensionality or the number of variables.
#' @param m An integer specifying the number of factors in the model.
#' @param distribution_type A character string indicating the type of distribution to use
#' for generating the data.
#' @return A list containing the following elements:
#' \item{data}{A numeric matrix of the generated data.}
#' \item{A}{A numeric matrix representing the factor loadings.}
#' \item{D}{A numeric matrix representing the uniquenesses, which is a diagonal matrix.}
#' @examples
#' n <- 1000
#' p <- 10
#' m <- 5
#' sigma1 <- 1
#' sigma2 <- matrix(c(1,0.7,0.7,1), 2, 2)
#' distribution_type <- "Asymmetric Log_Laplace"
#' results <- LFM(n, p, m, distribution_type)
#' print(results)
#' @export
#' @importFrom MASS mvrnorm
#' @importFrom LaplacesDemon rllaplace
#' @importFrom LaplacesDemon rallaplace
#' @importFrom LaplacesDemon rslaplace
#' @importFrom LaplacesDemon rlaplace
#' @importFrom LaplacesDemon lower.triangle
#' @importFrom LaplacesDemon is.positive.definite
#' @importFrom LaplacesDemon is.symmetric.matrix
#' @importFrom LaplacesDemon upper.triangle
#' @importFrom LaplacesDemon is.square.matrix
#' @importFrom matrixcalc frobenius.norm
#' @importFrom stats runif
#' @importFrom stats pnorm
#' @importFrom stats qnorm
#' @importFrom stats pcauchy
#' @importFrom stats qcauchy
LFM <- function(n, p, m, distribution_type) {
  mu <- t(matrix(rep(runif(p, 0, 1000), n), p, n))
  mu0 <- as.matrix(runif(m, 0))
  sigma0 <- diag(runif(m, 1))
  sigma1 <- 1
  sigma2 <- matrix(c(1, 0.7, 0.7, 1), 2, 2)
  F <- matrix(mvrnorm(n, mu0, sigma0), nrow = n)
  A <- matrix(runif(p * m, -1, 1), nrow = p)
  lanor <- numeric(0)
  if (distribution_type == "truncated_laplace") {
    mu1 <- c(0, 1)
    lower <- c(-2, -3)
    upper <- c(3, 3)
    truncated_normal <- function(n_samples, mean, sd, lower_bound, upper_bound) {
      p_lower <- pnorm(lower_bound, mean, sd)
      p_upper <- pnorm(upper_bound, mean, sd)
      u <- runif(n_samples, p_lower, p_upper)
      return(qnorm(u, mean, sd))
    }
    norm_dim1 <- truncated_normal(n * p / 2, mu1[1], 1, lower[1], upper[1])
    norm_dim2 <- truncated_normal(n * p / 2, mu1[2], 1, lower[2], upper[2])
    laplace_from_normal <- function(norm_values) {
      cauchy_values <- qcauchy(pnorm(norm_values, 0, 1), location = 0, scale = 1)
      return(cauchy_values * 0.7)
    }
    
    laplace_dim1 <- laplace_from_normal(norm_dim1)
    laplace_dim2 <- laplace_from_normal(norm_dim2)
    
    lanor <- c(laplace_dim1, laplace_dim2)
    
  } else if (distribution_type == "log_laplace") {
    lanor <- rllaplace(n * p, 0, sigma1)
  } else if (distribution_type == "Asymmetric Log_Laplace") {
    lanor <- rallaplace(n * p, 0, sigma1, 1)
  } else if (distribution_type == "Skew-Laplace") {
    lanor <- rslaplace(n * p, 0, 1, 1)
  } else {
    stop("Unknown distribution type")
  }
  
  epsilon <- matrix(lanor, nrow = n)
  D <- diag(t(epsilon) %*% epsilon)
  data <- mu + F %*% t(A) + epsilon
  
  return(list(data = data, A = A, D = D))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/generate_asymmetric_laplace.R
\name{generate_asymmetric_laplace}
\alias{generate_asymmetric_laplace}
\title{Generate Multivariate Asymmetric Laplace Random Vectors}
\usage{
generate_asymmetric_laplace(n, p, alpha, Sigma)
}
\arguments{
\item{n}{Number of observations (rows) to generate.}

\item{p}{Dimension of each observation (columns).}

\item{alpha}{A numeric vector of length \code{p} giving the asymmetry (shift) parameters for each coordinate. Can be a scalar if \code{p = 1}.}

\item{Sigma}{A \eqn{p \times p} positive definite covariance matrix for the Gaussian component \eqn{Z}.}
}
\value{
An \code{n x p} matrix where each row is a draw from the multivariate asymmetric Laplace distribution with shift vector \code{alpha} and scale matrix \code{Sigma}.
}
\description{
Generates a matrix of independent draws from a multivariate asymmetric Laplace distribution. Each row is \eqn{\epsilon = \alpha W + Z\sqrt{W}}, where \eqn{W \sim \text{Exp}(1)} independent of \eqn{Z \sim N_p(0, \Sigma)}. The parameter \eqn{\alpha} controls the asymmetry (skewness) in each coordinate.
}
\examples{
set.seed(123)
p <- 3
alpha <- c(0.5, -0.3, 0.2)
Sigma <- diag(p)
X <- generate_asymmetric_laplace(100, p, alpha, Sigma)
pairs(X)
}

\name{vehicle}
\alias{vehicle}
\docType{data}
\title{In Vehicle Coupon Recommendation Data}
\description{
This dataset contains information about coupon recommendations made to drivers in a vehicle, including various contextual features and the outcome of whether the coupon was accepted.
}
\usage{
vehicle
}
\format{
A data frame with multiple rows and 27 columns representing different features related to coupon recommendations.
\itemize{
\item \code{destination}: Driver's destination - No Urgent Place, Home, Work.
\item \code{passanger}: Passengers in the car - Alone, Friend(s), Kid(s), Partner.
\item \code{weather}: Current weather - Sunny, Rainy, Snowy.
\item \code{temperature}: Temperature in Fahrenheit - 55, 80, 30.
\item \code{time}: Time of day - 2PM, 10AM, 6PM, 7AM, 10PM.
\item \code{coupon}: Type of coupon - Restaurant(<$20), Coffee House, Carry out & Take away, Bar, Restaurant($20-$50).
\item \code{expiration}: Coupon expiration - 1d (1 day), 2h (2 hours).
\item \code{gender}: Driver's gender - Female, Male.
\item \code{age}: Driver's age group - 21, 46, 26, 31, 41, 50plus, 36, below21.
\item \code{maritalStatus}: Driver's marital status - Unmarried partner, Single, Married partner, Divorced, Widowed.
\item \code{has_Children}: Whether the driver has children - 1, 0.
\item \code{education}: Driver's education level - Some college - no degree, Bachelors degree, Associates degree, High School Graduate, Graduate degree (Masters or Doctorate), Some High School.
\item \code{occupation}: Driver's occupation - Various categories including Unemployed, Student, etc.
\item \code{income}: Driver's income range - Various ranges such as $37500 - $49999, $62500 - $74999, etc.
\item \code{Bar}: Frequency of bar visits per month - never, less1, 1~3, gt8, nan4~8.
\item \code{CoffeeHouse}: Frequency of coffeehouse visits per month - never, less1, 4~8, 1~3, gt8, nan.
\item \code{CarryAway}: Frequency of getting take-away food per month - n4~8, 1~3, gt8, less1, never.
\item \code{RestaurantLessThan20}: Frequency of visiting restaurants with average expense <$20 per month - 4~8, 1~3, less1, gt8, never.
\item \code{Restaurant20To50}: Frequency of visiting restaurants with average expense $20-$50 per month - 1~3, less1, never, gt8, 4~8, nan.
\item \code{toCoupon_GEQ15min}: Driving distance to the coupon location greater than 15 minutes - 0, 1.
\item \code{toCoupon_GEQ25min}: Driving distance to the coupon location greater than 25 minutes - 0, 1.
\item \code{direction_same}: Whether the coupon location is in the same direction as the current destination - 0, 1.
\item \code{direction_opp}: Whether the coupon location is in the opposite direction of the current destination - 1, 0.
\item \code{Y}: Whether the coupon was accepted - 1, 0.
}
}
\examples{
# Load the dataset
data(vehicle)

# Print the first few rows of the dataset
print(head(vehicle))

}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/DIPC.R
\name{DIPC}
\alias{DIPC}
\title{Distributed Incremental Principal Component Analysis (DIPC)}
\usage{
DIPC(data, m, eta, K)
}
\arguments{
\item{data}{Matrix of input data (n × p).}

\item{m}{Number of principal components.}

\item{eta}{Proportion of  initial batch to  total data  within each node.}

\item{K}{Number of nodes (distributed splits).}
}
\value{
List with per-node results and aggregated averages.
}
\description{
Apply IPC in a distributed manner across K nodes.
}
\examples{
library(LaplacesDemon)
library(MASS)
n=1000
p=10
m=5
mu=t(matrix(rep(runif(p,0,1000),n),p,n))
mu0=as.matrix(runif(m,0))
sigma0=diag(runif(m,1))
F=matrix(mvrnorm(n,mu0,sigma0),nrow=n)
A=matrix(runif(p*m,-1,1),nrow=p)
lanor <- rlaplace(n*p,0,1)
epsilon=matrix(lanor,nrow=n)
D=diag(t(epsilon)\%*\%epsilon)
data=mu+F\%*\%t(A)+epsilon
results <- DIPC(data, m, eta=0.8, K=5)
}

\name{bankruptcy}
\alias{bankruptcy}
\docType{data}

\title{Bankruptcy data}

\description{ The data set contain the ratio of retained earnings (RE) to total assets, and the ratio of earnings before interests and taxes (EBIT) to total assets of 66 American firms recorded in the form of ratios. Half of the selected firms had filed for bankruptcy. }

\usage{data(bankruptcy)}

\format{A data frame with the following variables:

\describe{
	\item{Y}{The status of the firm: \code{0} bankruptcy or \code{1} financially sound;}
	\item{RE}{Ratio of retained earnings to total assets;}
	\item{EBIT}{Ratio of earnings before interests and taxes to total assets}

}}
\examples{

data(bankruptcy)

}

\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/frobenius.norm.R
\name{frobenius.norm}
\alias{frobenius.norm}
\title{Compute the Frobenius Norm of a Matrix}
\usage{
frobenius.norm(M)
}
\arguments{
\item{M}{A numeric matrix.}
}
\value{
A non-negative scalar representing the Frobenius norm of \code{M}.
}
\description{
Calculates the Frobenius norm of a matrix, defined as the square root of the sum of squares of all its entries. This is the standard Euclidean norm when the matrix is treated as a vector.
}
\examples{
M <- matrix(1:6, nrow = 2)
frobenius.norm(M)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Dfactor.tests.R
\name{Dfactor.tests}
\alias{Dfactor.tests}
\title{Distributed Factor Model Testing with Wald, GRS, PY tests and FDR control}
\usage{
Dfactor.tests(ret, fac, n1, K, q.fdr = 0.05)
}
\arguments{
\item{ret}{A T × N matrix representing the excess returns of N assets at T time points.}

\item{fac}{A T × K matrix representing the returns of K factors at T time points.}

\item{n1}{The number of assets allocated to each node}

\item{K}{The number of nodes}

\item{q.fdr}{The significance level for FDR (False Discovery Rate) testing, defaulting to 5\%.}
}
\value{
A list containing the following components:
\item{alpha_list}{List of alpha vectors from each node}
\item{tstat_list}{List of t-statistics from each node}
\item{pval_list}{List of p-values from each node}
\item{Wald_list}{List of Wald test statistics from each node}
\item{p_Wald_list}{List of p-values for Wald tests from each node}
\item{GRS_list}{List of GRS test statistics from each node}
\item{p_GRS_list}{List of p-values for GRS tests from each node}
\item{PY_list}{List of Pesaran and Yamagata test statistics from each node}
\item{p_PY_list}{List of p-values for PY tests from each node}
\item{reject_fdr_list}{List of logical vectors indicating significant assets after FDR correction from each node}
\item{power_proxy_list}{List of number of significant assets after FDR correction from each node}
\item{combined_alpha}{Combined alpha vector from all nodes}
\item{combined_pval}{Combined p-value vector from all nodes}
\item{combined_reject_fdr}{Combined FDR rejection vector from all nodes}
\item{total_power_proxy}{Total number of significant assets across all nodes after FDR correction}
}
\description{
Performs comprehensive factor model testing in distributed environment across multiple nodes,
including joint tests (Wald, GRS, PY), individual asset t-tests, and False Discovery Rate control.
}
\examples{
set.seed(42)
T <- 120
N <- 100  # Larger dataset for distributed testing
K_factors <- 3
fac <- matrix(rnorm(T * K_factors), T, K_factors)
beta <- matrix(rnorm(N * K_factors), N, K_factors)
alpha <- rep(0, N)
alpha[1:10] <- 0.4 / 100  # 10 non-zero alphas
eps <- matrix(rnorm(T * N, sd = 0.02), T, N)
ret <- alpha + fac \%*\% t(beta) + eps

# Distributed testing with 4 nodes, each handling 25 assets
results <- Dfactor.tests(ret, fac, n1 = 25, K = 4, q.fdr = 0.05)

# View combined results
cat("Total significant assets after FDR:", results$total_power_proxy, "\n")
cat("Combined results across all nodes:\n")
print(summary(results$combined_alpha))

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/online_sir_lfm.R
\name{online_sir_lfm}
\alias{online_sir_lfm}
\title{Online Sufficient Dimension Reduction for Laplace Factor Model (LFM)}
\usage{
online_sir_lfm(
  X,
  K_true = NULL,
  K_max = NULL,
  c_robust = 1.345,
  eta = "auto",
  method = "gradient",
  verbose = FALSE
)
}
\arguments{
\item{X}{A matrix or data stream of size n x p (rows = observations, cols = features). Can be processed row-by-row in streaming setting.}

\item{K_true}{Optional true dimension (for monitoring). If NULL, will estimate online via BIC-like criterion.}

\item{K_max}{Maximum candidate dimension for online selection (default = min(10, ncol(X))).}

\item{c_robust}{Robustness scale for tanh transformation (default = 1.345, approx. 0.95 efficiency for Gaussian).}

\item{eta}{Learning rate schedule: either a function of t, or "auto" for 1/t.}

\item{method}{Optimization method: "gradient" for gradient-based updates with learning rate, or "perturbation" for direct eigenvector computation of the moment matrix (default = "gradient").}

\item{verbose}{Logical; if TRUE, prints progress and estimated K at each step.}
}
\value{
A list with:
\item{B_hat}{Final estimated basis matrix (p x K_est)}
\item{K_est}{Estimated structural dimension}
\item{B_path}{List of B estimates over time (optional, for debugging)}
\item{loss}{Reconstruction loss trace (optional)}
\item{method_used}{The optimization method actually used}
}
\description{
Implements an online SIR algorithm tailored for LFM data, using a proxy response constructed from the current subspace estimate and robust updates to handle heavy-tailed noise. The algorithm supports two optimization methods: gradient-based updates and perturbation-based updates.
}
\examples{
set.seed(123)
n <- 500; p <- 20; m <- 3
B_true <- qr.Q(qr(matrix(rnorm(p * m), p, m)))
f <- matrix(rnorm(n * m), n, m)
eps <- matrix(rexp(n * p, rate = 1) - 1, n, p) # Asymmetric Laplace-like noise
X <- f \%*\% t(B_true) + eps

# Using gradient method (default)
out_grad <- online_sir_lfm(X, K_true = m, verbose = TRUE)

# Using perturbation method
out_pert <- online_sir_lfm(X, K_true = m, method = "perturbation", verbose = TRUE)

}

\name{wholesale}
\alias{wholesale}
\docType{data}
\title{Wholesale Customers Data}
\description{
This dataset contains the annual spending amounts of wholesale customers on various product categories, along with their channel and region information.
}
\usage{wholesale}
\format{
A data frame with 440 rows and 8 columns.
\itemize{
\item \code{FRESH}: Annual spending (m.u.) on fresh products.
\item \code{MILK}: Annual spending (m.u.) on milk products.
\item \code{GROCERY}: Annual spending (m.u.) on grocery products.
\item \code{FROZEN}: Annual spending (m.u.) on frozen products.
\item \code{DETERGENTS_PAPER}: Annual spending (m.u.) on detergents and paper products.
\item \code{DELICATESSEN}: Annual spending (m.u.) on delicatessen products.
\item \code{CHANNEL}: Customers' channel - Horeca (Hotel/Restaurant/Café) or Retail channel (Nominal).
\item \code{REGION}: Customers' region - Lisbon, Oporto or Other (Nominal).
}
}
\examples{
# Load the dataset
data(wholesale)
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/generate_multivariate_laplace.R
\name{generate_multivariate_laplace}
\alias{generate_multivariate_laplace}
\title{Generate Multivariate Symmetric Laplace Random Vectors}
\usage{
generate_multivariate_laplace(n, p, Sigma_L)
}
\arguments{
\item{n}{Number of observations (rows) to generate.}

\item{p}{Dimension of each observation (columns).}

\item{Sigma_L}{A \eqn{p \times p} positive definite covariance matrix for the Gaussian component \eqn{Z}.}
}
\value{
An \code{n x p} matrix where each row is a draw from the multivariate symmetric Laplace distribution with scale matrix \eqn{\Sigma_L}.
}
\description{
Generates a matrix of independent draws from a multivariate symmetric Laplace distribution. Each row is an independent observation of a \eqn{p}-dimensional random vector \eqn{\epsilon = Z\sqrt{W}}, where \eqn{Z \sim N_p(0, \Sigma_L)} and \eqn{W \sim \text{Exp}(1)} independent of \eqn{Z}.
}
\examples{
set.seed(123)
p <- 3
Sigma_L <- diag(p)
X <- generate_multivariate_laplace(100, p, Sigma_L)
pairs(X)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Ftest.R
\name{Ftest}
\alias{Ftest}
\title{Apply the Farmtest method to the Laplace factor model}
\usage{
Ftest(
  data,
  p1,
  alpha = 0.05,
  K = -1,
  alternative = c("two.sided", "less", "greater")
)
}
\arguments{
\item{data}{A matrix or data frame of simulated or observed data from a Laplace factor model.}

\item{p1}{The number or proportion of non-zero hypotheses.}

\item{alpha}{The significance level for controlling the false discovery rate (default: 0.05).}

\item{K}{The number of factors to estimate (default: -1, meaning auto-detect).}

\item{alternative}{The alternative hypothesis: "two.sided", "less", or "greater" (default: "two.sided").}
}
\value{
A list containing the following elements:
\item{FDR}{The false discovery rate, which is the proportion of false positives among all discoveries (rejected hypotheses).}
\item{Power}{The statistical power of the test, which is the probability of correctly rejecting a false null hypothesis.}
\item{PValues}{A vector of p-values associated with each hypothesis test.}
\item{RejectedHypotheses}{The total number of hypotheses that were rejected by the FarmTest.}
\item{reject}{Indices of rejected hypotheses.}
\item{means}{Estimated means.}
}
\description{
This function simulates data from a Lapalce factor model and applies the FarmTest
for multiple hypothesis testing. It calculates the false discovery rate (FDR)
and power of the test.
}
\examples{
library(LaplacesDemon)
library(MASS)
n=1000
p=10
m=5
mu=t(matrix(rep(runif(p,0,1000),n),p,n))
mu0=as.matrix(runif(m,0))
sigma0=diag(runif(m,1))
F=matrix(mvrnorm(n,mu0,sigma0),nrow=n)
A=matrix(runif(p*m,-1,1),nrow=p)
lanor <- rlaplace(n*p,0,1)
epsilon=matrix(lanor,nrow=n)
D=diag(t(epsilon)\%*\%epsilon)
data=mu+F\%*\%t(A)+epsilon
p1=40
results <- Ftest(data, p1)
print(results$FDR)
print(results$Power)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/PPC.R
\name{PPC}
\alias{PPC}
\title{Projection principal component}
\usage{
PPC(data, m)
}
\arguments{
\item{data}{is a total  data set}

\item{m}{is the number of principal component}
}
\value{
Apro, Dpro, Sigmahatpro
}
\description{
Projection principal component
}
\examples{
library(LaplacesDemon)
library(MASS)
n=1000
p=10
m=5
mu=t(matrix(rep(runif(p,0,1000),n),p,n))
mu0=as.matrix(runif(m,0))
sigma0=diag(runif(m,1))
F=matrix(mvrnorm(n,mu0,sigma0),nrow=n)
A=matrix(runif(p*m,-1,1),nrow=p)
lanor <- rlaplace(n*p,0,1)
epsilon=matrix(lanor,nrow=n)
D=diag(t(epsilon)\%*\%epsilon)
data=mu+F\%*\%t(A)+epsilon
PPC(data=data,m=5)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{new_energy_vehicle}
\alias{new_energy_vehicle}
\title{New Energy Vehicle (NEV) Purchase Intention Survey Data}
\format{
A data frame with 520 rows and multiple variables:
\describe{
  \item{household_ice_owned}{Whether the household has purchased an internal-combustion (fuel) vehicle (single choice).}
  \item{household_nev_owned}{Whether the household has purchased a new energy vehicle (single choice).}

  \item{policy_subsidy_intention}{Effect of subsidy policies (e.g., toll exemptions, lower purchase price, low-interest loans) on NEV purchase intention (Likert 5-point).}
  \item{policy_license_intention}{Effect of license-plate policies (e.g., free registration, road-restriction privileges) on NEV purchase intention (Likert 5-point).}
  \item{environmental_intention}{Effect of environmental concerns on NEV purchase intention (Likert 5-point).}
  \item{infrastructure_intention}{Effect of charging infrastructure convenience on NEV purchase intention (Likert 5-point).}

  \item{driving_experience_factor}{Effect of driving experience (product factor) on NEV purchase intention (Likert 5-point).}
  \item{battery_performance_factor}{Effect of battery performance (range, lifespan, capacity, charging efficiency) on NEV purchase intention (Likert 5-point).}
  \item{safety_factor}{Effect of safety and technology maturity/reliability on NEV purchase intention (Likert 5-point).}

  \item{depreciation_cost_factor}{Effect of depreciation/durability concerns (economic factor) on NEV purchase intention (Likert 5-point).}
  \item{purchase_cost_factor}{Effect of purchase price (economic factor) on NEV purchase intention (Likert 5-point).}
  \item{charging_cost_factor}{Effect of charging cost (economic factor) on NEV purchase intention (Likert 5-point).}
  \item{maintenance_cost_factor}{Effect of maintenance/repair cost (economic factor) on NEV purchase intention (Likert 5-point).}

  \item{service_factor}{Effect of firm service (pre-sales and after-sales) on NEV purchase intention (Likert 5-point).}
  \item{brand_factor}{Effect of brand (firm factor) on NEV purchase intention (Likert 5-point).}
  \item{technology_advantage_factor}{Effect of perceived technological advantages (firm factor) on NEV purchase intention (Likert 5-point).}

  \item{purchase_intent}{Stated intention to purchase an NEV (Likert 5-point).}
  \item{recommend_intent}{Willingness to recommend NEVs to others (Likert 5-point).}
  \item{repurchase_intent}{Willingness to prioritize buying an NEV next time (Likert 5-point).}

  \item{gender}{Gender (single choice).}
  \item{age}{Age group (single choice).}
  \item{education}{Education level (single choice).}
  \item{occupation}{Occupation (single choice).}
  \item{hukou}{Household registration type (rural/urban; single choice).}
  \item{household_income}{Average monthly household income (categorical; single choice).}
}
}
\source{
Consumer survey dataset on NEV purchase intention and influencing factors.
}
\usage{
new_energy_vehicle
}
\description{
A questionnaire survey on consumers' purchase intention toward new energy vehicles (NEVs)
and its influencing factors. The dataset includes (i) household vehicle purchase history,
(ii) attitudes toward policy/product/economic/firm factors measured on a 5-point Likert scale,
and (iii) demographic information.
}
\details{
The Likert scale options are:
A = Strongly disagree, B = Disagree, C = Neutral, D = Agree, E = Strongly agree.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/factor.tests.R
\name{factor.tests}
\alias{factor.tests}
\title{Factor Model Testing with Wald, GRS, PY tests and FDR control}
\usage{
factor.tests(ret, fac, q.fdr = 0.05)
}
\arguments{
\item{ret}{A T × N matrix representing the excess returns of N assets at T time points.}

\item{fac}{A T × K matrix representing the returns of K factors at T time points.}

\item{q.fdr}{The significance level for FDR (False Discovery Rate) testing, defaulting to 5\%.}
}
\value{
A list containing the following components:
\item{alpha}{N-vector of estimated alphas for each asset}
\item{tstat}{N-vector of t-statistics for testing individual alphas}
\item{pval}{N-vector of p-values for individual alpha tests}
\item{Wald}{Wald test statistic for joint alpha significance}
\item{p_Wald}{p-value for Wald test}
\item{GRS}{GRS test statistic (finite-sample F-test)}
\item{p_GRS}{p-value for GRS test}
\item{PY}{Pesaran and Yamagata test statistic}
\item{p_PY}{p-value for PY test}
\item{reject_fdr}{Logical vector indicating which assets have significant alphas after FDR correction}
\item{fdr_p}{Adjusted p-values using Benjamini-Hochberg procedure}
\item{power_proxy}{Number of significant assets after FDR correction}
}
\description{
Performs comprehensive factor model testing including joint tests (Wald, GRS, PY), 
individual asset t-tests, and False Discovery Rate control.
}
\examples{
set.seed(42)
T <- 120
N <- 25
K <- 3
fac <- matrix(rnorm(T * K), T, K)
beta <- matrix(rnorm(N * K), N, K)
alpha <- rep(0, N)
alpha[1:3] <- 0.4 / 100  # 3 non-zero alphas
eps <- matrix(rnorm(T * N, sd = 0.02), T, N)
ret <- alpha + fac \%*\% t(beta) + eps
results <- factor.tests(ret, fac, q.fdr = 0.05)

# View results
cat("Wald test p-value:", results$p_Wald, "\n")
cat("GRS test p-value:", results$p_GRS, "\n")
cat("PY test p-value:", results$p_PY, "\n")
cat("Significant assets after FDR:", results$power_proxy, "\n")

}

\name{riboflavin}
\alias{riboflavin}
\docType{data}
\title{Riboflavin Production Data}
\description{
This dataset contains measurements of riboflavin (vitamin B2) production by Bacillus subtilis, a Gram-positive bacterium commonly used in industrial fermentation processes. The dataset includes
\eqn{n = 71} observations with \eqn{p = 4088} predictors, representing the logarithm of the expression levels of 4088 genes. The response variable is the log-transformed riboflavin production rate.
}
\usage{
data(riboflavin)
}
\format{
\describe{
\item{y}{Log-transformed riboflavin production rate (original name: \code{q_RIBFLV}). This is a continuous variable indicating the efficiency of riboflavin production by the bacterial strain.
}
\item{x}{A matrix of dimension \eqn{71 \times 4088} containing the logarithm of the expression levels of 4088 genes. Each column corresponds to a gene, and each row corresponds to an observation (experimental condition or time point).
}
}
}
\examples{
# Load the riboflavin dataset
data(riboflavin)

# Display the dimensions of the dataset
print(dim(riboflavin$x))
print(length(riboflavin$y))

}
\keyword{datasets}

\name{riboflavinv100}
\alias{riboflavinv100}
\docType{data}
\title{Riboflavin Production Data (Top 100 Genes)}
\description{
This dataset is a subset of the riboflavin production data by Bacillus subtilis, containing \eqn{n = 71} observations. It includes the response variable (log-transformed riboflavin production rate) and the 100 genes with the largest empirical variances from the original dataset.
}
\usage{
data(riboflavinv100)
}
\format{
\describe{
\item{y}{Log-transformed riboflavin production rate (original name: \code{q_RIBFLV}). This is a continuous variable indicating the efficiency of riboflavin production by the bacterial strain.
}
\item{x}{A matrix of dimension \eqn{71 \times 100} containing the logarithm of the expression levels of the 100 genes with the largest empirical variances.
}
}
}
\examples{
# Load the riboflavinv100 dataset
data(riboflavinv100)

# Display the dimensions of the dataset
print(dim(riboflavinv100$x))
print(length(riboflavinv100$y))

}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/osdr_lfm.R
\name{osdr_lfm}
\alias{osdr_lfm}
\title{Online Sufficient Dimension Reduction for Laplace Factor Models (OSDR-LFM)}
\usage{
osdr_lfm(
  X,
  Y = NULL,
  laplace_type = c("symmetric", "asymmetric", "skewed"),
  K_max = NULL,
  H = NULL,
  method_svd = c("gradient", "perturbation"),
  is_distributed = FALSE,
  node_id = 1,
  sync_interval = 50,
  verbose = FALSE
)
}
\arguments{
\item{X}{numeric matrix (n x p), observations in rows.}

\item{Y}{optional numeric vector (n) of proxy responses (e.g., factor scores). 
If NULL, uses norm of projection as proxy (unsupervised LFM mode).}

\item{laplace_type}{character; one of "symmetric", "asymmetric", or "skewed".}

\item{K_max}{integer; maximum candidate dimension (default = min(10, p)).}

\item{H}{integer; number of slices for SIR (default = max(5, floor(sqrt(n)))).}

\item{method_svd}{character; "perturbation" or "gradient" (default = "gradient").}

\item{is_distributed}{logical; if TRUE, simulate distributed node behavior.}

\item{node_id}{integer; node identifier (only used if is_distributed = TRUE).}

\item{sync_interval}{integer; how often to "aggregate" in distributed mode (ignored if not distributed).}

\item{verbose}{logical; print progress.}
}
\value{
list with B_hat (p x K_est), K_est, lambda_trace, and (if distributed) local_B.
}
\description{
Implements an online SIR-based sufficient dimension reduction method tailored for Laplace Factor Models (LFM) with symmetric, asymmetric, or skewed error structures.
Supports distributed deployment via local updates and global aggregation.
}
\examples{
set.seed(42)
n <- 600; p <- 30; m <- 4
A <- qr.Q(qr(matrix(rnorm(p * m), p, m)))
F <- matrix(rnorm(n * m), n, m)
eps <- matrix(rexp(n * p) - rexp(n * p), n, p)
X <- F \%*\% t(A) + eps

out <- osdr_lfm(X, laplace_type = "asymmetric", K_max = 6, verbose = TRUE)
cat("Estimated K:", out$K_est, "\n")

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/FanPC.R
\name{FanPC}
\alias{FanPC}
\title{Apply the FanPC method to the Laplace factor model}
\usage{
FanPC(data, m)
}
\arguments{
\item{data}{A matrix of input data.}

\item{m}{is the number of principal component}
}
\value{
AF,DF,SigmahatF
}
\description{
This function performs Factor Analysis via Principal Component (FanPC) on a given data set. It calculates the estimated factor loading matrix (AF), specific variance matrix (DF), and the mean squared errors.
}
\examples{
library(LaplacesDemon)
library(MASS)
n=1000
p=10
m=5
mu=t(matrix(rep(runif(p,0,1000),n),p,n))
mu0=as.matrix(runif(m,0))
sigma0=diag(runif(m,1))
F=matrix(mvrnorm(n,mu0,sigma0),nrow=n)
A=matrix(runif(p*m,-1,1),nrow=p)
lanor <- rlaplace(n*p,0,1)
epsilon=matrix(lanor,nrow=n)
D=diag(t(epsilon)\%*\%epsilon)
data=mu+F\%*\%t(A)+epsilon
results <- FanPC(data, m)
print(results)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/DPC.R
\name{DPC}
\alias{DPC}
\title{Distributed principal component}
\usage{
DPC(data, m, n1, K)
}
\arguments{
\item{data}{is a total data set}

\item{m}{is the number of principal component}

\item{n1}{is  the length of each data subset}

\item{K}{is the number of nodes}
}
\value{
Ahat,Dhat,Sigmahathat
}
\description{
Distributed principal component
}
\examples{
library(LaplacesDemon)
library(MASS)
n=1000
p=10
m=5
mu=t(matrix(rep(runif(p,0,1000),n),p,n))
mu0=as.matrix(runif(m,0))
sigma0=diag(runif(m,1))
F=matrix(mvrnorm(n,mu0,sigma0),nrow=n)
A=matrix(runif(p*m,-1,1),nrow=p)
lanor <- rlaplace(n*p,0,1)
epsilon=matrix(lanor,nrow=n)
D=diag(t(epsilon)\%*\%epsilon)
data=mu+F\%*\%t(A)+epsilon
DPC(data,m=3,n1=128,K=2)
}

\name{review}
\alias{review}
\docType{data}
\title{Review}
\description{
This dataset contains travel reviews from TripAdvisor.com, covering destinations in 11 categories across East Asia. Each traveler's rating is mapped to a scale from Terrible (0) to Excellent (4), and the average rating for each category per user is provided.
}
\usage{
review
}
\format{
A data frame with multiple rows and 12 columns.
\itemize{
\item \code{User_ID}: Unique identifier for each user (Categorical).
\item \code{Art_Galleries}: Average user feedback on art galleries.
\item \code{Dance_Clubs}: Average user feedback on dance clubs.
\item \code{Juice_Bars}: Average user feedback on juice bars.
\item \code{Restaurants}: Average user feedback on restaurants.
\item \code{Museums}: Average user feedback on museums.
\item \code{Resorts}: Average user feedback on resorts.
\item \code{Parks_Picnic_Spots}: Average user feedback on parks and picnic spots.
\item \code{Beaches}: Average user feedback on beaches.
\item \code{Theaters}: Average user feedback on theaters.
\item \code{Religious_Institutions}: Average user feedback on religious institutions.
}
}
\examples{
# Load the dataset
data(review)

# Print the first few rows of the dataset
print(head(review))
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/calculate_DAA.R
\name{calculate_DAA}
\alias{calculate_DAA}
\title{Distance Between Column Spaces of Two Loading Matrices}
\usage{
calculate_DAA(Ahat, A1, m, p)
}
\arguments{
\item{Ahat}{Estimated factor loading matrix of dimension \code{p x m}, or at least having \code{m} columns.}

\item{A1}{True or reference factor loading matrix of dimension \code{p x m} (or \code{p x m1}, but typically \code{m} columns).}

\item{m}{Number of factors (structural dimension) used in the scaling factor.}

\item{p}{Number of variables (observed dimensions).}
}
\value{
A scalar in \eqn{[0,1]} representing the discrepancy between the column spaces of \code{Ahat} and \code{A1}. A value of 0 indicates identical subspaces; values closer to 1 indicate greater discrepancy.
}
\description{
Computes a normalized distance between the subspaces spanned by the columns of two factor loading matrices \code{Ahat} (estimated) and \code{A1} (true/reference). The metric is based on the projection matrix of \code{A1}, regularized to avoid singularity, and is scaled by the number of factors \code{m} and squared number of variables \code{p^2} to lie in \eqn{[0,1]}.
}
\details{
The distance is defined as
\deqn{D(A_{\mathrm{est}}, A_{\mathrm{true}}) = \sqrt{1 - \frac{1}{m p^2} \, \mathrm{Re}\left( \mathrm{tr}\left( A_{\mathrm{est}}^\top (A_{\mathrm{true}} A_{\mathrm{true}}^\top + \epsilon I)^{-1} A_{\mathrm{true}} A_{\mathrm{true}}^\top A_{\mathrm{est}} \right) \right)},}
where \eqn{\epsilon = 10^{-6}} is a small ridge parameter to ensure invertibility of \eqn{A_{\mathrm{true}} A_{\mathrm{true}}^\top}.
}
\examples{
set.seed(123)
p <- 10; m <- 3
# True loading matrix (orthonormal columns)
A_true <- qr.Q(qr(matrix(rnorm(p * m), p, m)))
# Slightly perturbed estimate
A_est <- A_true + matrix(rnorm(p * m, sd = 0.05), p, m)
A_est <- qr.Q(qr(A_est))  # re-orthogonalize

calculate_DAA(A_est, A_true, m, p)

}

\name{protein}
\alias{protein}
\docType{data}
\title{Protein Secondary Structure Data}
\description{
This dataset contains protein sequences and their corresponding secondary structures, including beta-sheets (E), helices (H), and coils (_).
}
\usage{
protein
}
\format{
A data frame with multiple rows and columns representing protein sequences and their secondary structures.
\itemize{
\item \code{Sequence}: Amino acid sequence (using 3-letter codes).
\item \code{Structure}: Secondary structure of the protein (E for beta-sheet, H for helix, _ for coil).
\item \code{Parameters}: Additional parameters for neural networks (to be ignored).
\item \code{Biophysical_Constants}: Biophysical constants (to be ignored).
}
}
\examples{
# Load the dataset
data(protein)

# Print the first few rows of the dataset
print(head(protein))
}
\keyword{datasets}

\name{Wine}
\alias{Wine}
\title{Wine Data}
\usage{data(Wine)}
\description{
The Wine dataset contains the results of a chemical analysis of wines grown in the same region in Italy but derived from three different cultivars. The analysis determined the quantities of 13 constituents found in each of the three types of wines. This dataset is commonly used for classification tasks to determine the origin of wines based on their chemical properties.
}
\format{
A data frame with 178 rows and 14 columns representing different features of wines.
\itemize{
\item \code{Class}: Categorical target variable indicating the type of wine (1, 2, or 3).
\item \code{Alcohol}: Continuous feature representing the alcohol content.
\item \code{Malic_acid}: Continuous feature representing the malic acid content.
\item \code{Ash}: Continuous feature representing the ash content.
\item \code{Alcalinity_of_ash}: Continuous feature representing the alcalinity of ash.
\item \code{Magnesium}: Integer feature representing the magnesium content.
\item \code{Total_phenols}: Continuous feature representing the total phenols content.
\item \code{Flavanoids}: Continuous feature representing the flavanoids content.
\item \code{Nonflavanoid_phenols}: Continuous feature representing the nonflavanoid phenols content.
\item \code{Proanthocyanins}: Continuous feature representing the proanthocyanins content.
\item \code{Color_intensity}: Continuous feature representing the color intensity.
\item \code{Hue}: Continuous feature representing the hue.
\item \code{OD280_OD315_of_diluted_wines}: Continuous feature representing the OD280/OD315 of diluted wines.
\item \code{Proline}: Continuous feature representing the proline content.
}
}
\examples{
# Load the dataset
data(Wine)

# Print the first few rows of the dataset
print(head(Wine))
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/DSAPC.R
\name{DSAPC}
\alias{DSAPC}
\title{The distributed stochastic approximation principal component for handling online data sets with highly correlated data across multiple nodes.}
\usage{
DSAPC(data, m, eta, n1, K)
}
\arguments{
\item{data}{is a highly correlated online data set}

\item{m}{is the number of principal component}

\item{eta}{is the proportion of online data to total data}

\item{n1}{is the length of each data subset}

\item{K}{is the number of nodes}
}
\value{
Asa, Dsa (lists containing results from each node)
}
\description{
The distributed stochastic approximation principal component for handling online data sets with highly correlated data across multiple nodes.
}
\examples{
library(LaplacesDemon)
library(MASS)
n=1000
p=10
m=5
mu=t(matrix(rep(runif(p,0,1000),n),p,n))
mu0=as.matrix(runif(m,0))
sigma0=diag(runif(m,1))
F=matrix(mvrnorm(n,mu0,sigma0),nrow=n)
A=matrix(runif(p*m,-1,1),nrow=p)
lanor <- rlaplace(n*p,0,1)
epsilon=matrix(lanor,nrow=n)
D=diag(t(epsilon)\%*\%epsilon)
data=mu+F\%*\%t(A)+epsilon
DSAPC(data=data, m=3, eta=0.8, n1=128, K=2)
}

\name{Heart}
\alias{Heart}
\title{Heart}
\usage{data(Heart)}
\description{
This dataset contains information about heart disease diagnosis, including various clinical attributes and the presence of heart disease in patients. The dataset is commonly used for classification tasks to predict the presence of heart disease.
}
\format{
A data frame with multiple rows and 14 columns representing different features related to heart disease diagnosis.
\itemize{
\item \code{age}: Age in years (integer).
\item \code{sex}: Sex (1 = male; 0 = female) (categorical).
\item \code{cp}: Chest pain type (categorical).
\item \code{trestbps}: Resting blood pressure (in mm Hg on admission to the hospital) (integer).
\item \code{chol}: Serum cholesterol in mg/dl (integer).
\item \code{fbs}: Fasting blood sugar > 120 mg/dl (1 = true; 0 = false) (categorical).
\item \code{restecg}: Resting electrocardiographic results (categorical).
\item \code{thalach}: Maximum heart rate achieved (integer).
\item \code{exang}: Exercise-induced angina (1 = yes; 0 = no) (categorical).
\item \code{oldpeak}: ST depression induced by exercise relative to rest (integer).
\item \code{slope}: The slope of the peak exercise ST segment (categorical).
\item \code{ca}: Number of major vessels (0-3) colored by fluoroscopy (integer).
\item \code{thal}: Thalassemia (3 = normal; 6 = fixed defect; 7 = reversible defect) (categorical).
\item \code{num}: Diagnosis of heart disease (angiographic disease status) (integer).
}
}
\examples{
# Load the dataset
data(Heart)

# Print the first few rows of the dataset
print(head(Heart))
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/generate_skew_laplace.R
\name{generate_skew_laplace}
\alias{generate_skew_laplace}
\title{Generate Multivariate Skew Laplace Random Vectors}
\usage{
generate_skew_laplace(n, p, alpha, gamma, Sigma)
}
\arguments{
\item{n}{Number of observations (rows) to generate.}

\item{p}{Dimension of each observation (columns).}

\item{alpha}{Numeric vector of length \code{p} giving the location (shift) parameter. Can be a scalar if \code{p = 1}.}

\item{gamma}{Numeric vector of length \code{p} giving the skewness parameter for each coordinate. Can be a scalar if \code{p = 1}.}

\item{Sigma}{A \eqn{p \times p} positive definite covariance matrix for the Gaussian component \eqn{Z}.}
}
\value{
An \code{n x p} matrix where each row is a draw from the specified multivariate skew Laplace distribution.
}
\description{
Generates a matrix of independent draws from a multivariate skew Laplace distribution with location vector \eqn{\alpha}, scale matrix \eqn{\Sigma}, and skewness parameter \eqn{\gamma}. Each observation is constructed as \eqn{\epsilon = \alpha + \gamma V + \sqrt{V} Z}, where \eqn{V \sim \text{Gamma}((p+1)/2, 1/2)} and \eqn{Z \sim N_p(0, \Sigma)} independent of \eqn{V}.
}
\examples{
set.seed(123)
p <- 3
alpha <- c(0.1, -0.2, 0.0)
gamma <- c(0.5, 0.3, -0.4)
Sigma <- diag(p)
X <- generate_skew_laplace(100, p, alpha, gamma, Sigma)
pairs(X)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/estimate_AD_from_OSDR.R
\name{estimate_AD_from_OSDR}
\alias{estimate_AD_from_OSDR}
\title{Estimate Factor Loadings and Uniquenesses from OSDR Output}
\usage{
estimate_AD_from_OSDR(data, res, m)
}
\arguments{
\item{data}{The original data matrix (n x p) used in the online SIR procedure.}

\item{res}{A list returned by \code{\link{online_sir_lfm}}, containing at least the component \code{B_hat} (p x K_est matrix).}

\item{m}{The target number of factors (structural dimension). If \code{B_hat} has fewer columns, the missing directions are filled with zeros; if more, only the first \code{m} columns are retained.}
}
\value{
A list with components:
\item{A_est}{Estimated factor loading matrix of dimension p x m.}
\item{D_est}{Estimated diagonal noise covariance matrix of dimension p x p, with diagonal entries forced to be at least 0.01.}
}
\description{
Extracts the estimated factor loading matrix \eqn{A} and the diagonal noise covariance matrix \eqn{D} from the output of \code{\link{online_sir_lfm}}. The subspace estimate \code{B_hat} is truncated or padded to have exactly \code{m} columns, used as \eqn{A}, and \eqn{D} is obtained by subtracting the factor covariance contribution from the sample covariance matrix, with a lower bound on the diagonal entries to ensure positivity.
}
\examples{
\dontrun{
set.seed(123)
n <- 500; p <- 20; m <- 3
B_true <- qr.Q(qr(matrix(rnorm(p * m), p, m)))
f <- matrix(rnorm(n * m), n, m)
eps <- matrix(rexp(n * p, rate = 1) - 1, n, p) # Laplace noise
X <- f \%*\% t(B_true) + eps

# Run online SIR
res <- online_sir_lfm(X, K_true = m, method = "gradient", verbose = FALSE)

# Estimate A and D from the result
AD <- estimate_AD_from_OSDR(X, res, m)
print(head(AD$A_est))
print(diag(AD$D_est)[1:5])
}

}

\name{Breast}
\alias{Breast}
\title{Breast}
\usage{data(Breast)}
\description{
This dataset contains original clinical cases reported by Dr. Wolberg. The data are grouped chronologically, reflecting the time periods when the samples were collected. The dataset includes various attributes related to breast cancer diagnosis.
}
\format{
A data frame with 699 rows and several columns representing different features related to breast cancer diagnosis.
\itemize{
\item \code{Sample_code_number}: Identification number for the sample.
\item \code{Clump_Thickness}: 1-10
\item \code{Uniformity_of_Cell_Size}: 1-10
\item \code{Uniformity_of_Cell_Shape}: 1-10
\item \code{Marginal_Adhesion}: 1-10
\item \code{Single_Epithelial_Cell_Size}: 1-10
\item \code{Bare_Nuclei}: 1-10 (some values may be missing or revised)
\item \code{Bland_Chromatin}: 1-10
\item \code{Normal_Nucleoli}: 1-10
\item \code{Mitoses}: 1-10
\item \code{Class}: 2 (benign) or 4 (malignant)
}
}
\examples{
# Load the dataset
data(Breast)

# Print the first few rows of the dataset
print(head(Breast))
}
\keyword{datasets}

\name{concrete}
\alias{concrete}
\docType{data}
\title{Concrete Slump Test Data}
\description{
This dataset contains measurements related to the slump test of concrete, including input variables (concrete ingredients) and output variables (slump, flow, and compressive strength).
}
\usage{
concrete
}
\format{
A data frame with 103 rows and 10 columns.
\itemize{
\item \code{Cement}: Amount of cement (kg in one M^3 concrete).
\item \code{Slag}: Amount of slag (kg in one M^3 concrete).
\item \code{Fly_ash}: Amount of fly ash (kg in one M^3 concrete).
\item \code{Water}: Amount of water (kg in one M^3 concrete).
\item \code{SP}: Amount of superplasticizer (kg in one M^3 concrete).
\item \code{Coarse_Aggr}: Amount of coarse aggregate (kg in one M^3 concrete).
\item \code{Fine_Aggr}: Amount of fine aggregate (kg in one M^3 concrete).
\item \code{SLUMP}: Slump of the concrete (cm).
\item \code{FLOW}: Flow of the concrete (cm).
\item \code{Compressive_Strength}: 28-day compressive strength of the concrete (MPa).
}
}
\examples{
# Load the dataset
data(concrete)

# Print the first few rows of the dataset
print(head(concrete))

}
\keyword{datasets}

\name{Sonar}
\alias{Sonar}
\title{Sonar}
\usage{data(Sonar)}
\description{
This dataset contains sonar signals bounced off a metal cylinder (mines) and a roughly cylindrical rock. The task is to classify whether the signal is from a mine or a rock based on the sonar signal patterns.
}
\format{
A data frame with 208 rows and 61 columns representing different features of sonar signals.
\itemize{
\item \code{Attribute1}: Continuous feature representing energy within a frequency band.
\item \code{Attribute2}: Continuous feature representing energy within a frequency band.
\item \code{Attribute3}: Continuous feature representing energy within a frequency band.
\item \code{...}: Additional continuous features (up to Attribute60).
\item \code{Class}: Categorical target variable ('M' for mine, 'R' for rock).
}
}
\examples{
# Load the dataset
data(Sonar)

# Print the first few rows of the dataset
print(head(Sonar))
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/LFM.R
\name{LFM}
\alias{LFM}
\title{Generate Laplace factor models}
\usage{
LFM(n, p, m, distribution_type)
}
\arguments{
\item{n}{An integer specifying the sample size.}

\item{p}{An integer specifying the sample dimensionality or the number of variables.}

\item{m}{An integer specifying the number of factors in the model.}

\item{distribution_type}{A character string indicating the type of distribution to use
for generating the data.}
}
\value{
A list containing the following elements:
\item{data}{A numeric matrix of the generated data.}
\item{A}{A numeric matrix representing the factor loadings.}
\item{D}{A numeric matrix representing the uniquenesses, which is a diagonal matrix.}
}
\description{
The function is to generate Laplace factor model data.
The function supports various distribution types for generating the data,
including:
- `truncated_laplace`: Truncated Laplace distribution
- `log_laplace`: Univariate Symmetric Log-Laplace distribution
- `Asymmetric Log_Laplace`: Log-Laplace distribution
- `Skew-Laplace`: Skew-Laplace distribution
}
\examples{
n <- 1000
p <- 10
m <- 5
sigma1 <- 1
sigma2 <- matrix(c(1,0.7,0.7,1), 2, 2)
distribution_type <- "Asymmetric Log_Laplace"
results <- LFM(n, p, m, distribution_type)
print(results)
}

\name{yacht_hydrodynamics}
\alias{yacht_hydrodynamics}
\docType{data}
\title{Yacht Hydrodynamics Data}
\description{
This dataset contains the hydrodynamic characteristics of sailing yachts, including design parameters and performance metrics.
}
\usage{
yacht_hydrodynamics
}
\format{
A data frame with 308 rows and 7 columns.
\itemize{
\item \code{Residuary Resistance}: Residuary resistance per unit weight of displacement (performance metric).
\item \code{Longitudinal Position of Center of Buoyancy}: Longitudinal position of the center of buoyancy.
\item \code{Prismatic Coefficient}: Prismatic coefficient.
\item \code{Length-Displacement Ratio}: Length-displacement ratio.
\item \code{Beam-Draft Ratio}: Beam-draft ratio.
\item \code{Length-Beam Ratio}: Length-beam ratio.
\item \code{Froude Number}: Froude number.
}
}
\examples{
# Load the dataset
data(yacht_hydrodynamics)

# Print the first few rows of the dataset
print(head(yacht_hydrodynamics))

}
\keyword{datasets}

\name{Australian}
\alias{Australian}
\title{Australian}
\usage{data(Australian)}
\description{
This dataset contains information about credit card applications. All attribute names and values have been changed to meaningless symbols to protect confidentiality. The dataset includes a mix of continuous and categorical attributes, with some missing values.
}
\format{
A data frame with 690 rows and 15 columns representing different features related to credit card applications.
\itemize{
\item \code{A1}: Categorical - 0, 1 (formerly: a, b)
\item \code{A2}: Continuous
\item \code{A3}: Continuous
\item \code{A4}: Categorical - 1, 2, 3 (formerly: p, g, gg)
\item \code{A5}: Categorical - 1 to 14 (formerly: ff, d, i, k, j, aa, m, c, w, e, q, r, cc, x)
\item \code{A6}: Categorical - 1 to 9 (formerly: ff, dd, j, bb, v, n, o, h, z)
\item \code{A7}: Continuous
\item \code{A8}: Categorical - 1, 0 (formerly: t, f)
\item \code{A9}: Categorical - 1, 0 (formerly: t, f)
\item \code{A10}: Continuous
\item \code{A11}: Categorical - 1, 0 (formerly: t, f)
\item \code{A12}: Categorical - 1, 2, 3 (formerly: s, g, p)
\item \code{A13}: Continuous
\item \code{A14}: Continuous
\item \code{A15}: Class attribute - 1, 2 (formerly: +, -)
}
}
\examples{
# Load the dataset
data(Australian)

# Print the first few rows of the dataset
print(head(Australian))
}
\keyword{datasets}

\name{ionosphere}
\alias{ionosphere}
\title{ionosphere Data}
\usage{data(ionosphere)}
\description{
This dataset contains radar returns from the ionosphere, collected by a system in Goose Bay, Labrador. The dataset is used for classifying radar returns as 'good' or 'bad' based on the presence of structure in the ionosphere.
}
\format{
A data frame with multiple rows and 35 columns representing different features related to radar returns.
\itemize{
\item \code{Attribute1}: Continuous feature.
\item \code{Attribute2}: Continuous feature.
\item \code{Attribute3}: Continuous feature.
\item \code{Attribute4}: Continuous feature.
\item \code{Attribute5}: Continuous feature.
\item \code{Attribute6}: Continuous feature.
\item \code{Attribute7}: Continuous feature.
\item \code{Attribute8}: Continuous feature.
\item \code{Attribute9}: Continuous feature.
\item \code{Attribute10}: Continuous feature.
\item \code{...}: Additional continuous features (up to Attribute34).
\item \code{Class}: Binary classification target ('good' or 'bad').
}
}
\examples{
# Load the dataset
data(ionosphere)

# Print the first few rows of the dataset
print(head(ionosphere))
}
\keyword{datasets}

\name{Iris}
\alias{Iris}
\title{Iris Data}
\usage{data(Iris)}
\description{
The Iris dataset is a classic and widely-used dataset in the field of machine learning and statistics. It contains measurements of sepal length, sepal width, petal length, and petal width for three species of iris plants. The dataset is commonly used for classification tasks.
}
\format{
A data frame with 150 rows and 5 columns representing different features of iris plants.
\itemize{
\item \code{Sepal.Length}: Sepal length in centimeters (continuous).
\item \code{Sepal.Width}: Sepal width in centimeters (continuous).
\item \code{Petal.Length}: Petal length in centimeters (continuous).
\item \code{Petal.Width}: Petal width in centimeters (continuous).
\item \code{Species}: Species of iris plant (categorical): Iris Setosa, Iris Versicolor, or Iris Virginica.
}
}
\examples{
# Load the dataset
data(Iris)

# Print the first few rows of the dataset
print(head(Iris))
}
\keyword{datasets}
